from __future__ import annotations
from typing import Any

from checkov.common.util.consts import START_LINE, END_LINE


def generate_resource_key_recursive(conf: dict[str, Any] | list[str] | str, key: str, start_line: int,
                                    end_line: int) -> str:
    if not isinstance(conf, dict):
        return key

    for k, value in conf.items():
        if isinstance(value, dict) and value[START_LINE] <= start_line <= end_line <= value[END_LINE]:
            next_key = f'{key}.{k}' if key else k
            return generate_resource_key_recursive(value, next_key, start_line, end_line)
        if isinstance(value, list):
            return f'{key}.{k}' if key else k
        if isinstance(value, str):
            return key

    return key



from checkov.common.models.enums import CheckCategories
from checkov.terraform.checks.resource.base_resource_negative_value_check import BaseResourceNegativeValueCheck


class GKEDisabledLegacyAuth(BaseResourceNegativeValueCheck):
    def __init__(self):
        name = "Ensure Legacy Authorization is set to Disabled on Kubernetes Engine Clusters"
        id = "CKV_GCP_7"
        supported_resources = ['google_container_cluster']
        categories = [CheckCategories.KUBERNETES]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def get_inspected_key(self):
        return 'enable_legacy_abac'

    def get_forbidden_values(self):
        return [True]


check = GKEDisabledLegacyAuth()



from abc import abstractmethod
from collections.abc import Iterable
from typing import Callable, Optional, Dict, Any

from checkov.cloudformation.checks.resource.registry import cfn_registry
from checkov.common.checks.base_check import BaseCheck
from checkov.common.models.enums import CheckCategories, CheckResult
from checkov.common.multi_signature import multi_signature


class BaseResourceCheck(BaseCheck):
    def __init__(
        self,
        name: str,
        id: str,
        categories: "Iterable[CheckCategories]",
        supported_resources: "Iterable[str]",
        guideline: Optional[str] = None,
    ) -> None:
        super().__init__(
            name=name,
            id=id,
            categories=categories,
            supported_entities=supported_resources,
            block_type="resource",
            guideline=guideline,
        )
        self.supported_resources = supported_resources
        cfn_registry.register(self)

    def scan_entity_conf(self, conf: Dict[str, Any], entity_type: str) -> CheckResult:
        self.entity_type = entity_type

        return self.scan_resource_conf(conf, entity_type)

    @multi_signature()
    @abstractmethod
    def scan_resource_conf(self, conf: Dict[str, Any], entity_type: str) -> CheckResult:
        raise NotImplementedError()

    @classmethod
    @scan_resource_conf.add_signature(args=["self", "conf"])
    def _scan_resource_conf_self_conf(cls, wrapped: Callable[..., CheckResult]) -> Callable[..., CheckResult]:
        def wrapper(self: BaseCheck, conf: Dict[str, Any], entity_type: Optional[str] = None) -> CheckResult:
            # keep default argument for entity_type so old code, that doesn't set it, will work.
            return wrapped(self, conf)

        return wrapper



from typing import Optional, Any, Dict, List
from checkov.common.checks_infra.solvers.attribute_solvers.base_attribute_solver import BaseAttributeSolver
from checkov.common.graph.checks_infra.enums import Operators
from checkov.common.util.type_forcers import force_int


class RangeIncludesAttributeSolver(BaseAttributeSolver):
    operator = Operators.RANGE_INCLUDES  # noqa: CCE003  # a static attribute

    def __init__(
        self, resource_types: List[str], attribute: Optional[str], value: Any, is_jsonpath_check: bool = False
    ) -> None:
        super().__init__(resource_types, attribute, force_int(value), is_jsonpath_check)

    def _get_operation(self, vertex: Dict[str, Any], attribute: Optional[str]) -> bool:
        attr = vertex.get(attribute)  # type:ignore[arg-type]  # due to attribute can be None

        if attr is None:
            return False

        # expects one of the following values:
        # - an actual int
        # - a string that parses to an int
        # - *
        # - a string range like '1000-2000'

        if attr == '*':
            return True

        if isinstance(attr, str) and attr.count("-") == 1:
            try:
                start, end = attr.split("-")
                return True if force_int(start) <= self.value <= force_int(end) else False
            except ValueError:
                # Occurs if there are not two entries or if one is not an int, in which case we just give up
                return False

        return True if force_int(attr) == self.value else False



from checkov.common.models.enums import CheckCategories
from checkov.terraform.checks.resource.base_resource_negative_value_check import BaseResourceNegativeValueCheck


class ObjectStoragePublic(BaseResourceNegativeValueCheck):
    def __init__(self):
        name = "Ensure OCI Object Storage is not Public"
        id = "CKV_OCI_10"
        supported_resources = ['oci_objectstorage_bucket']
        categories = [CheckCategories.GENERAL_SECURITY]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def get_inspected_key(self):
        return "access_type"

    def get_forbidden_values(self):
        return ["ObjectRead", "ObjectReadWithoutList"]


check = ObjectStoragePublic()



from __future__ import annotations

from typing import Any

from checkov.common.models.enums import CheckCategories, CheckResult
from checkov.terraform.checks.resource.base_resource_negative_value_check import BaseResourceNegativeValueCheck


class AKSUpgradeChannel(BaseResourceNegativeValueCheck):
    def __init__(self) -> None:
        name = "Ensure AKS cluster upgrade channel is chosen"
        id = "CKV_AZURE_171"
        supported_resources = ("azurerm_kubernetes_cluster",)
        categories = (CheckCategories.NETWORKING,)
        super().__init__(
            name=name,
            id=id,
            categories=categories,
            supported_resources=supported_resources,
            missing_attribute_result=CheckResult.FAILED,
        )

    def get_inspected_key(self) -> str:
        return "automatic_channel_upgrade"

    def get_forbidden_values(self) -> list[Any]:
        return ["none"]


check = AKSUpgradeChannel()



from __future__ import annotations
from typing import Optional, List, Tuple, Dict, Any

from checkov.common.bridgecrew.severities import Severity

from checkov.common.output.record import Record
from checkov.common.typing import _CheckResult


class SecretsRecord(Record):
    def __init__(self,
                 check_id: str,
                 check_name: str,
                 check_result: _CheckResult,
                 code_block: List[Tuple[int, str]],
                 file_path: str,
                 file_line_range: List[int],
                 resource: str,
                 evaluations: Optional[Dict[str, Any]],
                 check_class: str,
                 file_abs_path: str,
                 entity_tags: Optional[Dict[str, str]] = None,
                 caller_file_path: Optional[str] = None,
                 caller_file_line_range: Optional[Tuple[int, int]] = None,
                 bc_check_id: Optional[str] = None,
                 resource_address: Optional[str] = None,
                 severity: Optional[Severity] = None,
                 bc_category: Optional[str] = None,
                 benchmarks: dict[str, list[str]] | None = None,
                 description: Optional[str] = None,
                 short_description: Optional[str] = None,
                 vulnerability_details: Optional[Dict[str, Any]] = None,
                 connected_node: Optional[Dict[str, Any]] = None,
                 details: Optional[List[str]] = None,
                 check_len: int | None = None,
                 definition_context_file_path: Optional[str] = None,
                 validation_status: Optional[str] = None
                 ):
        super().__init__(check_id=check_id,
                         check_name=check_name,
                         check_result=check_result,
                         code_block=code_block,
                         file_path=file_path,
                         file_line_range=file_line_range,
                         resource=resource,
                         evaluations=evaluations,
                         check_class=check_class,
                         file_abs_path=file_abs_path,
                         entity_tags=entity_tags,
                         bc_check_id=bc_check_id,
                         severity=severity,
                         details=details,
                         caller_file_path=caller_file_path,
                         caller_file_line_range=caller_file_line_range,
                         resource_address=resource_address,
                         bc_category=bc_category,
                         benchmarks=benchmarks,
                         description=description,
                         short_description=short_description,
                         vulnerability_details=vulnerability_details,
                         connected_node=connected_node,
                         check_len=check_len,
                         definition_context_file_path=definition_context_file_path
                         )
        self.validation_status = validation_status



from enum import Enum

from checkov.common.bridgecrew.code_categories import CodeCategoryType


class CustomerSubscription(str, Enum):
    IAC = "IAC"
    SCA = "SCA"
    SECRETS = "SECRETS"


class BillingPlan(str, Enum):
    DEVELOPER_BASED = "DEVELOPER_BASED"
    RESOURCE_BASED = "RESOURCE_BASED"


SubscriptionCategoryMapping = {
    CustomerSubscription.IAC: (CodeCategoryType.IAC, CodeCategoryType.SUPPLY_CHAIN),
    CustomerSubscription.SCA: (CodeCategoryType.OPEN_SOURCE, CodeCategoryType.IMAGES),
    CustomerSubscription.SECRETS: (CodeCategoryType.SECRETS,)
}

CategoryToSubscriptionMapping = {}
for sub, cats in SubscriptionCategoryMapping.items():
    for cat in cats:
        CategoryToSubscriptionMapping[cat] = sub


open_source_categories = [CodeCategoryType.IAC, CodeCategoryType.SECRETS, CodeCategoryType.SUPPLY_CHAIN]



from typing import Dict, Any

from checkov.common.models.enums import CheckResult
from checkov.kubernetes.checks.resource.base_container_check import BaseK8sContainerCheck


class ApiServerBasicAuthFile(BaseK8sContainerCheck):
    def __init__(self) -> None:
        id = "CKV_K8S_69"
        name = "Ensure that the --basic-auth-file argument is not set"
        super().__init__(name=name, id=id)

    def scan_container_conf(self, metadata: Dict[str, Any], conf: Dict[str, Any]) -> CheckResult:
        self.evaluated_container_keys = ["command"]
        command = conf.get("command")
        if isinstance(command, list):
            if "kube-apiserver" in command:
                if any(x.startswith("--basic-auth-file") for x in command):
                    return CheckResult.FAILED

        return CheckResult.PASSED


check = ApiServerBasicAuthFile()



from __future__ import annotations

import logging
from typing import Any, TypeVar

_T = TypeVar("_T")


def get_inner_dict(source_dict: dict[str, Any], path_as_list: list[str]) -> dict[str, Any]:
    result = source_dict
    for index in path_as_list:
        result = result[index]
    return result


def merge_dicts(*dicts: dict[_T, Any]) -> dict[_T, Any]:
    """
    Merges two or more dicts. If there are duplicate keys, later dict arguments take precedence.

    Null, empty, or non-dict arguments are qiuetly skipped.
    :param dicts:
    :return:
    """
    res: dict[Any, Any] = {}
    for d in dicts:
        if not d or not isinstance(d, dict):
            continue
        res = {**res, **d}
    return res


def search_deep_keys(
    search_text: str, obj: dict[str, Any] | list[dict[str, Any]] | None, path: list[int | str]
) -> list[list[int | str]]:
    """Search deep for keys and get their values"""
    keys: list[list[int | str]] = []
    if isinstance(obj, dict):
        for key in obj:
            pathprop = path[:]
            pathprop.append(key)
            if key == search_text:
                pathprop.append(obj[key])
                keys.append(pathprop)
                # pop the last element off for nesting of found elements for
                # dict and list checks
                pathprop = pathprop[:-1]
            if isinstance(obj[key], dict):
                if key != 'parent_metadata':
                    # Don't go back to the parent metadata, it is scanned for the parent
                    keys.extend(search_deep_keys(search_text, obj[key], pathprop))
            elif isinstance(obj[key], list):
                for index, item in enumerate(obj[key]):
                    pathproparr = pathprop[:]
                    pathproparr.append(index)
                    keys.extend(search_deep_keys(search_text, item, pathproparr))
    elif isinstance(obj, list):
        for index, item in enumerate(obj):
            pathprop = path[:]
            pathprop.append(index)
            keys.extend(search_deep_keys(search_text, item, pathprop))

    return keys


def find_in_dict(input_dict: dict[str, Any], key_path: str) -> Any:
    """Tries to retrieve the value under the given 'key_path', otherwise returns None."""

    value: Any = input_dict
    key_list = key_path.split("/")

    try:
        for key in key_list:
            if key.startswith("[") and key.endswith("]"):
                if isinstance(value, list):
                    idx = int(key[1:-1])
                    value = value[idx]
                    continue
                else:
                    return None

            value = value.get(key)
            if value is None:
                return None
    except (AttributeError, IndexError, KeyError, TypeError, ValueError):
        logging.debug(f"Could not find {key_path} in dict")
        return None

    return value


def get_empty_list_str() -> list[str]:
    """Returns an empty list with type 'list[str]'

    This is needed for using empty lists with a list union type hint
    ex.
        foo: list[str] | list[int] = []

    more info can be found here https://github.com/python/mypy/issues/6463
    """

    return []



from __future__ import annotations

import json
from ast import literal_eval
import logging
import os
import re
from collections.abc import Hashable
from copy import deepcopy
from json import JSONDecodeError

import dpath.util
from typing import TYPE_CHECKING, List, Dict, Any, Tuple, Union, Optional

from lark.tree import Tree

from checkov.common.graph.graph_builder import Edge
from checkov.common.graph.graph_builder.utils import join_trimmed_strings
from checkov.common.graph.graph_builder.variable_rendering.renderer import VariableRenderer
from checkov.common.util.type_forcers import force_int
from checkov.common.graph.graph_builder.graph_components.attribute_names import CustomAttributes, reserved_attribute_names
from checkov.terraform.graph_builder.graph_components.block_types import BlockType
from checkov.terraform.graph_builder.utils import (
    get_referenced_vertices_in_value,
    remove_index_pattern_from_str,
    attribute_has_nested_attributes, attribute_has_dup_with_dynamic_attributes,
)
from checkov.terraform.graph_builder.variable_rendering.vertex_reference import VertexReference
from checkov.terraform.graph_builder.variable_rendering.evaluate_terraform import replace_string_value, \
    evaluate_terraform

if TYPE_CHECKING:
    from checkov.terraform.graph_builder.local_graph import TerraformLocalGraph

VAR_TYPE_DEFAULT_VALUES: dict[str, list[Any] | dict[str, Any]] = {
    'list': [],
    'map': {}
}

DYNAMIC_STRING = 'dynamic'
DYNAMIC_BLOCKS_LISTS = 'list'
DYNAMIC_BLOCKS_MAPS = 'map'
FOR_LOOP = 'for'
LOOKUP = 'lookup'
DOT_SEPERATOR = '.'
LEFT_BRACKET_WITH_QUOTATION = '["'
RIGHT_BRACKET_WITH_QUOTATION = '"]'
LEFT_BRACKET = '['
RIGHT_BRACKET = ']'

# matches the internal value of the 'type' attribute: usually like '${map}' or '${map(string)}', but could possibly just
# be like 'map' or 'map(string)' (but once we hit a ( or } we can stop)
TYPE_REGEX = re.compile(r'^(\${)?([a-z]+)')
CHECKOV_RENDER_MAX_LEN = force_int(os.getenv("CHECKOV_RENDER_MAX_LEN", "10000"))


class TerraformVariableRenderer(VariableRenderer):
    def __init__(self, local_graph: "TerraformLocalGraph") -> None:
        super().__init__(local_graph)

    def attributes_no_eval(self, attribute: str, vertex_index: int) -> bool:
        """
        Check if the attribute should not be evaluated.
        :param attribute: the attribute to check
        :param vertex_index: the index of the current vertex
        :return bool: True if the attribute should not be evaluated and False otherwise
        """
        if attribute in {"template_body", "template"}:
            return True

        # OCI policy statements have a special syntax and should not be evaluated.
        # Check if the vertex at this index is an OCI terraform resource.
        if attribute == "statements":
            vertex_attributes = self.local_graph.get_vertex_attributes_by_index(vertex_index)
            if vertex_attributes and vertex_attributes.get("resource_type", "").startswith("oci_"):
                return True

        return False

    def evaluate_vertex_attribute_from_edge(self, edge_list: List[Edge]) -> None:
        multiple_edges = len(edge_list) > 1
        edge = edge_list[0]
        origin_vertex_attributes = self.local_graph.vertices[edge.origin].attributes
        val_to_eval = deepcopy(origin_vertex_attributes.get(edge.label, ""))

        referenced_vertices = get_referenced_vertices_in_value(
            value=val_to_eval, aliases={}, resources_types=self.local_graph.get_resources_types_in_graph()
        )
        if not referenced_vertices:
            origin_vertex = self.local_graph.vertices[edge.origin]
            destination_vertex = self.local_graph.vertices[edge.dest]
            if origin_vertex.block_type == BlockType.VARIABLE and destination_vertex.block_type == BlockType.MODULE:
                self.update_evaluated_value(
                    changed_attribute_key=edge.label,
                    changed_attribute_value=destination_vertex.attributes[origin_vertex.name],
                    vertex=edge.origin,
                    change_origin_id=edge.dest,
                    attribute_at_dest=edge.label,
                )
                return
            if (
                origin_vertex.block_type == BlockType.VARIABLE
                and destination_vertex.block_type == BlockType.TF_VARIABLE
            ):
                destination_vertex = list(filter(lambda v: v.block_type == BlockType.TF_VARIABLE, map(lambda e: self.local_graph.vertices[e.dest], edge_list)))[-1]  # evaluate the last specified variable based on .tfvars precedence
                self.update_evaluated_value(
                    changed_attribute_key=edge.label,
                    changed_attribute_value=destination_vertex.attributes["default"],
                    vertex=edge.origin,
                    change_origin_id=edge.dest,
                    attribute_at_dest=edge.label,
                )
                return

        modified_vertex_attributes = self.local_graph.vertices[edge.origin].attributes
        origin_val = modified_vertex_attributes.get(edge.label, "")
        val_to_eval = deepcopy(origin_val)
        first_key_path = None

        if referenced_vertices:
            for edge in edge_list:
                dest_vertex_attributes = self.local_graph.get_vertex_attributes_by_index(edge.dest, add_hash=False)
                key_path_in_dest_vertex, replaced_key = self.find_path_from_referenced_vertices(
                    referenced_vertices, dest_vertex_attributes
                )
                if not key_path_in_dest_vertex or not replaced_key:
                    continue
                if not first_key_path:
                    first_key_path = key_path_in_dest_vertex

                evaluated_attribute_value = self.extract_value_from_vertex(
                    key_path_in_dest_vertex, dest_vertex_attributes
                )
                if evaluated_attribute_value is not None:
                    val_to_eval = self.replace_value(edge, val_to_eval, replaced_key, evaluated_attribute_value, True)
                if not multiple_edges and val_to_eval != origin_val:
                    self.update_evaluated_value(
                        changed_attribute_key=edge.label,
                        changed_attribute_value=val_to_eval,
                        vertex=edge.origin,
                        change_origin_id=edge.dest,
                        attribute_at_dest=key_path_in_dest_vertex,
                    )

        if multiple_edges and val_to_eval != origin_val:
            self.update_evaluated_value(
                changed_attribute_key=edge.label,
                changed_attribute_value=val_to_eval,
                vertex=edge.origin,
                change_origin_id=edge.dest,
                attribute_at_dest=first_key_path,
            )

        # Avoid loops on output => output edges
        if (
            self.local_graph.vertices[edge.origin].block_type == BlockType.OUTPUT
            and self.local_graph.vertices[edge.dest].block_type == BlockType.OUTPUT
        ):
            if edge.origin not in self.done_edges_by_origin_vertex:
                self.done_edges_by_origin_vertex[edge.origin] = []
            self.done_edges_by_origin_vertex[edge.origin].append(edge)

    def extract_value_from_vertex(self, key_path: List[str], attributes: Dict[str, Any]) -> Any:
        for i, _ in enumerate(key_path):
            key = join_trimmed_strings(char_to_join=".", str_lst=key_path, num_to_trim=i)
            value = attributes.get(key, None)
            if value is not None:
                return value

        reversed_key_path = key_path[::-1]
        for i, _ in enumerate(reversed_key_path):
            key = join_trimmed_strings(char_to_join=".", str_lst=reversed_key_path, num_to_trim=i)
            value = attributes.get(key, None)
            if value is not None:
                return value

        if attributes.get(CustomAttributes.BLOCK_TYPE) in (BlockType.VARIABLE, BlockType.TF_VARIABLE):
            var_type = attributes.get('type')
            default_val = attributes.get("default")
            if default_val is None:
                # this allows functions like merge(var.xyz, ...) to work even with no default value
                default_val = self.get_default_placeholder_value(var_type)
            value = None
            if isinstance(default_val, dict):
                value = self.extract_value_from_vertex(key_path, default_val)
            elif (
                isinstance(var_type, str)
                and var_type.startswith("${object")
                and isinstance(default_val, str)
            ):
                try:
                    default_val_eval = literal_eval(default_val)
                    if isinstance(default_val_eval, dict):
                        value = self.extract_value_from_vertex(key_path, default_val_eval)
                except Exception:
                    logging.debug(f"cant evaluate this rendered value: {default_val}")
            return default_val if not value else value
        if attributes.get(CustomAttributes.BLOCK_TYPE) == BlockType.OUTPUT:
            return attributes.get("value")
        return None

    @staticmethod
    def get_default_placeholder_value(var_type: Any) -> list[Any] | dict[str, Any] | None:
        if not var_type or not isinstance(var_type, str):
            return None
        match = TYPE_REGEX.match(var_type)
        return VAR_TYPE_DEFAULT_VALUES.get(match.group(2)) if match else None

    @staticmethod
    def find_path_from_referenced_vertices(
        referenced_vertices: List[VertexReference], vertex_attributes: Dict[str, Any]
    ) -> Tuple[List[str], str]:
        """
        :param referenced_vertices: an array of VertexReference
        :param vertex_attributes: attributes to search
        :return attribute_path: [] if referenced_vertices does not contain vertex_attributes,
                                else the path to the searched attribute: ['vpc_id']
        :return origin_value
        """
        for vertex_reference in referenced_vertices:
            block_type = vertex_reference.block_type
            attribute_path = vertex_reference.sub_parts
            copy_of_attribute_path = attribute_path.copy()
            if vertex_attributes[CustomAttributes.BLOCK_TYPE] == block_type:
                for i, _ in enumerate(copy_of_attribute_path):
                    copy_of_attribute_path[i] = remove_index_pattern_from_str(copy_of_attribute_path[i])
                    name = ".".join(copy_of_attribute_path[: i + 1])
                    if vertex_attributes[CustomAttributes.BLOCK_NAME] == name:
                        return attribute_path, vertex_reference.origin_value
            elif block_type == BlockType.MODULE:
                copy_of_attribute_path.reverse()
                for i, _ in enumerate(copy_of_attribute_path):
                    copy_of_attribute_path[i] = remove_index_pattern_from_str(copy_of_attribute_path[i])
                    name = ".".join(copy_of_attribute_path[: i + 1])
                    if vertex_attributes[CustomAttributes.BLOCK_NAME] == name:
                        return name.split("."), vertex_reference.origin_value
        return [], ""

    def update_evaluated_value(
        self,
        changed_attribute_key: str,
        changed_attribute_value: Union[str, List[str]],
        vertex: int,
        change_origin_id: int,
        attribute_at_dest: Optional[Union[str, List[str]]] = None,
    ) -> None:
        """
        The function updates the value of changed_attribute_key with changed_attribute_value for vertex
        """
        str_to_evaluate = (
            str(changed_attribute_value)
            if self.attributes_no_eval(changed_attribute_key, vertex)
            else f'"{str(changed_attribute_value)}"'
        )
        str_to_evaluate = str_to_evaluate.replace("\\\\", "\\")
        evaluated_attribute_value = (
            str_to_evaluate if self.attributes_no_eval(changed_attribute_key, vertex) else evaluate_terraform(str_to_evaluate)
        )
        self.local_graph.update_vertex_attribute(
            vertex, changed_attribute_key, evaluated_attribute_value, change_origin_id, attribute_at_dest
        )

    def evaluate_vertices_attributes(self) -> None:
        for vertex in self.local_graph.vertices:
            decoded_attributes = vertex.get_attribute_dict(add_hash=False)
            for attr in decoded_attributes:
                if attr in vertex.changed_attributes:
                    continue
                origin_value = decoded_attributes[attr]
                if not isinstance(origin_value, str):
                    continue
                evaluated_attribute_value = evaluate_terraform(origin_value)
                if origin_value != evaluated_attribute_value:
                    vertex.update_inner_attribute(attr, vertex.attributes, evaluated_attribute_value)

    def replace_value(
        self,
        edge: Edge,
        original_val: List[Any],
        replaced_key: str,
        replaced_value: Any,
        keep_origin: bool,
        count: int = 0,
    ) -> Union[Any, List[Any]]:
        if count > 1:
            return original_val
        new_val = replace_string_value(
            original_str=original_val,
            str_to_replace=replaced_key,
            replaced_value=replaced_value,
            keep_origin=keep_origin,
        )
        return new_val

    def _render_variables_from_vertices(self) -> None:
        self._render_dynamic_blocks()

    def _render_dynamic_blocks(self) -> None:
        vertex_indices = self.local_graph.vertices_by_block_type[BlockType.RESOURCE]

        for idx in vertex_indices:
            vertex = self.local_graph.vertices[idx]
            if vertex.has_dynamic_block:
                # only check dynamic blocks on the root level for now
                dynamic_blocks = vertex.attributes.get("dynamic")
                if dynamic_blocks:
                    try:
                        rendered_blocks = self._process_dynamic_blocks(dynamic_blocks)
                    except Exception:
                        logging.info(f'Failed to process dynamic blocks in file {vertex.path} of resource {vertex.name}'
                                     f' for blocks: {dynamic_blocks}', exc_info=True)
                        continue
                    changed_attributes = []

                    for block_name, block_confs in rendered_blocks.items():
                        vertex.update_inner_attribute(block_name, vertex.attributes, block_confs)
                        changed_attributes.append(block_name)

                    self.local_graph.update_vertex_config(vertex, changed_attributes, True)

    @staticmethod
    def _extract_dynamic_arguments(block_name: str, block_content: Dict[str, Any], dynamic_arguments: List[str],
                                   path_accumulator: List[str]) -> None:
        dynamic_value_dot_ref = f"{block_name}.value"
        dynamic_value_bracket_ref = f'{block_name}["value"]'
        dynamic_value_refs = (dynamic_value_dot_ref, dynamic_value_bracket_ref)
        for argument, value in block_content.items():
            if value in dynamic_value_refs or isinstance(value, str) and dynamic_value_dot_ref in value:
                dynamic_arguments.append(DOT_SEPERATOR.join(filter(None, [*path_accumulator, argument])))
            elif isinstance(value, dict):
                TerraformVariableRenderer._extract_dynamic_arguments(block_name, value, dynamic_arguments,
                                                                     path_accumulator + [argument])

    @staticmethod
    def _process_dynamic_blocks(dynamic_blocks: list[dict[str, Any]] | dict[str, Any]) -> dict[
            str, list[dict[str, Any]]]:
        rendered_blocks: dict[str, list[dict[str, Any]] | dict[str, Any]] = {}

        if not isinstance(dynamic_blocks, list) and not isinstance(dynamic_blocks, dict):
            logging.info(f"Dynamic blocks found, but of type {type(dynamic_blocks)}")

        dynamic_type = DYNAMIC_BLOCKS_LISTS
        if isinstance(dynamic_blocks, dict):
            dynamic_blocks = [dynamic_blocks]
            dynamic_type = DYNAMIC_BLOCKS_MAPS

        for block in dynamic_blocks:
            block_name, block_values = next(iter(block.items()))  # only one block per dynamic_block
            block_content = block_values.get("content")
            dynamic_values = block_values.get("for_each")
            dynamic_values = TerraformVariableRenderer._handle_for_loop_in_dynamic_values(dynamic_values)
            if not block_content or not dynamic_values or isinstance(dynamic_values, str):
                continue

            dynamic_arguments = []
            TerraformVariableRenderer._extract_dynamic_arguments(block_name, block_content, dynamic_arguments, [])
            if dynamic_arguments and isinstance(dynamic_values, list):
                block_confs = []
                for dynamic_value in dynamic_values:
                    block_conf = deepcopy(block_content)
                    block_conf.pop(DYNAMIC_STRING, None)
                    for dynamic_argument in dynamic_arguments:
                        if dynamic_type == DYNAMIC_BLOCKS_MAPS:
                            if not isinstance(dynamic_value, dict):
                                continue
                            TerraformVariableRenderer._assign_dynamic_value_for_list(
                                dynamic_value=dynamic_value,
                                dynamic_argument=dynamic_argument,
                                block_conf=block_conf,
                                block_content=block_content,
                                block_name=block_name
                            )

                        else:
                            TerraformVariableRenderer._assign_dynamic_value_for_map(
                                dynamic_value=dynamic_value,
                                dynamic_argument=dynamic_argument,
                                block_conf=block_conf,
                                block_content=block_content,
                            )

                    block_confs.append(block_conf)
                rendered_blocks[block_name] = block_confs if len(block_confs) > 1 else block_confs[0]

            if DYNAMIC_STRING in block_content and dynamic_values:
                try:
                    next_key = next(iter(block_content[DYNAMIC_STRING].keys()))
                except (StopIteration, AttributeError):
                    continue
                block_content[DYNAMIC_STRING][next_key]['for_each'] = dynamic_values

                try:
                    flatten_key = next(iter(rendered_blocks.keys()))
                except StopIteration:
                    flatten_key = ''
                if rendered_blocks.get(flatten_key) and next_key in rendered_blocks[flatten_key]:
                    rendered_blocks[flatten_key].update(TerraformVariableRenderer._process_dynamic_blocks(block_content[DYNAMIC_STRING]))
                elif isinstance(rendered_blocks.get(flatten_key), list) and isinstance(dynamic_values, list):
                    for i in range(len(rendered_blocks[flatten_key])):
                        block_content[DYNAMIC_STRING][next_key]['for_each'] = [dynamic_values[i]]
                        rendered_blocks[flatten_key][i].update(TerraformVariableRenderer._process_dynamic_blocks(block_content[DYNAMIC_STRING]))
                else:
                    rendered_blocks.update(TerraformVariableRenderer._process_dynamic_blocks(block_content[DYNAMIC_STRING]))

        return rendered_blocks

    @staticmethod
    def _assign_dynamic_value_for_list(
            dynamic_value: str | dict[str, Any] | dict[str, list[dict[str, dict[str, Any]]]],
            dynamic_argument: str,
            block_conf: dict[str, Any],
            block_content: dict[str, Any],
            block_name: str,
    ):
        dynamic_value_in_map = TerraformVariableRenderer.extract_dynamic_value_in_map(
            dpath.get(block_content, dynamic_argument, separator=DOT_SEPERATOR), dynamic_argument
        )
        if block_name not in dynamic_value and dynamic_value_in_map in dynamic_value:
            dpath.set(block_conf, dynamic_argument, dynamic_value[dynamic_value_in_map], separator=DOT_SEPERATOR)
        else:
            try:
                if DOT_SEPERATOR in dynamic_argument:
                    dynamic_args = dynamic_argument.split(DOT_SEPERATOR)
                    dpath.set(block_conf, dynamic_argument, dynamic_value[block_name][0][dynamic_args[0]][dynamic_args[1]], separator=DOT_SEPERATOR)
                else:
                    dpath.set(block_conf, dynamic_argument, dynamic_value[block_name][0][dynamic_value_in_map], separator=DOT_SEPERATOR)
            except (KeyError, IndexError):
                if block_content.get(dynamic_argument) and LOOKUP in block_content.get(dynamic_argument):
                    block_conf[dynamic_argument] = get_lookup_value(block_content, dynamic_argument)
                else:
                    return

    @staticmethod
    def _handle_for_loop_in_dynamic_values(dynamic_values: str | dict[str, Any]) -> str | dict[str, Any] | list[dict[str, Any]]:
        if not isinstance(dynamic_values, str):
            return dynamic_values

        if (dynamic_values.startswith(LEFT_BRACKET + FOR_LOOP) or dynamic_values.startswith(LEFT_BRACKET + " " + FOR_LOOP)) and dynamic_values.endswith(RIGHT_BRACKET):
            rendered_dynamic_values = dynamic_values[1:-1]
            start_bracket_idx = rendered_dynamic_values.find(LEFT_BRACKET)
            end_bracket_idx = find_match_bracket_index(rendered_dynamic_values, start_bracket_idx)
            if start_bracket_idx != -1 and end_bracket_idx != -1:
                rendered_dynamic_values = rendered_dynamic_values[start_bracket_idx:end_bracket_idx + 1].replace("'", '"')
            try:
                return json.loads(rendered_dynamic_values)
            except JSONDecodeError:
                return dynamic_values
        return dynamic_values

    @staticmethod
    def _assign_dynamic_value_for_map(
            dynamic_value: str | dict[str, Any],
            dynamic_argument: str,
            block_conf: dict[str, Any],
            block_content: dict[str, Any],
    ):
        if isinstance(dynamic_value, dict):
            if dynamic_argument in dynamic_value:
                dpath.set(block_conf, dynamic_argument, dynamic_value[dynamic_argument], separator=DOT_SEPERATOR)
            else:
                if isinstance(block_content, dict) and dynamic_argument in block_content and isinstance(block_content[dynamic_argument], str):
                    lookup_value = get_lookup_value(block_content, dynamic_argument)
                    dpath.set(block_conf, dynamic_argument, lookup_value, separator=DOT_SEPERATOR)
        else:
            dpath.set(block_conf, dynamic_argument, dynamic_value, separator=DOT_SEPERATOR)

    def evaluate_non_rendered_values(self) -> None:
        for index, vertex in enumerate(self.local_graph.vertices):
            changed_attributes = {}
            attributes: Dict[str, Any] = {}
            vertex.get_origin_attributes(attributes)
            filtered_attributes = [
                attr
                for attr in vertex.attributes
                if attr not in reserved_attribute_names and not attribute_has_nested_attributes(attr, vertex.attributes)
                and not attribute_has_dup_with_dynamic_attributes(attr, vertex.attributes)
            ]
            for attribute in filtered_attributes:
                curr_val = vertex.attributes.get(attribute)
                lst_curr_val = curr_val
                if not isinstance(lst_curr_val, list):
                    lst_curr_val = [lst_curr_val]
                if len(lst_curr_val) > 0 and isinstance(lst_curr_val[0], Tree):
                    lst_curr_val[0] = str(lst_curr_val[0])
                evaluated_lst = []
                for inner_val in lst_curr_val:
                    if (
                        isinstance(inner_val, str)
                        and not any(c in inner_val for c in ("{", "}", "[", "]", "="))
                        or self.attributes_no_eval(attribute, index)
                    ):
                        evaluated_lst.append(inner_val)
                        continue
                    evaluated = self.evaluate_value(inner_val)
                    evaluated_lst.append(evaluated)
                evaluated = evaluated_lst
                if not isinstance(curr_val, list):
                    evaluated = evaluated_lst[0]
                if evaluated != curr_val:
                    vertex.update_inner_attribute(attribute, vertex.attributes, evaluated)
                    changed_attributes[attribute] = evaluated
            self.local_graph.update_vertex_config(vertex, changed_attributes)

    @staticmethod
    def extract_dynamic_value_in_map(dynamic_value: str, dynamic_argument: str = '') -> str:
        if LOOKUP in dynamic_value and dynamic_argument in dynamic_value:
            return dynamic_argument

        dynamic_value_in_map = dynamic_value.split(DOT_SEPERATOR)[-1]
        if LEFT_BRACKET not in dynamic_value_in_map:
            return dynamic_value_in_map
        return dynamic_value_in_map.split(LEFT_BRACKET_WITH_QUOTATION)[-1].replace(RIGHT_BRACKET_WITH_QUOTATION, '')

    def evaluate_value(self, val: Any) -> Any:
        val_length: int = len(str(val))
        if CHECKOV_RENDER_MAX_LEN and 0 < CHECKOV_RENDER_MAX_LEN < val_length:
            logging.debug(f'Rendering was skipped for a {val_length}-character-long string. If you wish to have it '
                          f'evaluated, please set the environment variable CHECKOV_RENDER_MAX_LEN '
                          f'to {str(val_length + 1)} or to 0 to allow rendering of any length')
            return val
        if type(val) not in [str, list, set, dict]:
            evaluated_val = val
        elif isinstance(val, str):
            evaluated_val = evaluate_terraform(val, keep_interpolations=False)
        elif isinstance(val, list):
            evaluated_val = []
            for v in val:
                evaluated_val.append(self.evaluate_value(v))
        elif isinstance(val, set):
            evaluated_val = set()
            for v in val:
                evaluated_v = self.evaluate_value(v)
                if isinstance(evaluated_v, Hashable):
                    evaluated_val.add(evaluated_v)
                else:
                    evaluated_val.add(str(evaluated_v))
        else:
            evaluated_val = {}
            for k, v in val.items():
                evaluated_key = self.evaluate_value(k)
                evaluated_val[evaluated_key] = self.evaluate_value(v)
        return evaluated_val


def find_match_bracket_index(s: str, open_bracket_idx: int) -> int:
    res = {}
    pstack = []
    for i, c in enumerate(s):
        if c == LEFT_BRACKET:
            pstack.append(i)
        elif c == RIGHT_BRACKET:
            if len(pstack) == 0:
                logging.debug("No matching closing brackets at: " + str(i))
                return -1
            res[pstack.pop()] = i

    if len(pstack) > 0:
        logging.debug("No matching opening brackets at: " + str(pstack.pop()))

    return res.get(open_bracket_idx) or -1


def get_lookup_value(block_content, dynamic_argument) -> str:
    lookup_value: str = ''
    if 'None' in block_content[dynamic_argument]:
        lookup_value = 'null'
    elif 'False' in block_content[dynamic_argument]:
        lookup_value = 'false'
    elif 'True' in block_content[dynamic_argument]:
        lookup_value = 'true'
    return lookup_value



from checkov.common.models.enums import CheckCategories
from checkov.terraform.checks.resource.base_resource_value_check import BaseResourceValueCheck


class SpringCloudAPIPortalHTTPSOnly(BaseResourceValueCheck):
    def __init__(self) -> None:
        name = "Ensures Spring Cloud API Portal is enabled on for HTTPS"
        id = "CKV_AZURE_161"
        supported_resources = ("azurerm_spring_cloud_api_portal",)
        categories = (CheckCategories.NETWORKING,)
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def get_inspected_key(self) -> str:
        return "https_only_enabled"


check = SpringCloudAPIPortalHTTPSOnly()



from __future__ import annotations

from typing import Any

from checkov.common.models.enums import CheckCategories, CheckResult
from checkov.terraform.checks.resource.base_resource_check import BaseResourceCheck

INSECURE_TLS_VERSIONS = ("None", "TLS10")


class CDNTLSProtocol12(BaseResourceCheck):
    def __init__(self) -> None:
        # This is the full description of your check
        description = "Ensure the Azure CDN endpoint is using the latest version of TLS encryption"

        # This is the Unique ID for your check
        id = "CKV_AZURE_200"

        # These are the Terraform objects supported by this check (ex: aws_iam_policy_document)
        supported_resources = ("azurerm_cdn_endpoint_custom_domain",)

        # Valid CheckCategories are defined in checkov/common/models/enums.py
        categories = (CheckCategories.NETWORKING,)
        super().__init__(name=description, id=id, categories=categories, supported_resources=supported_resources)

    def scan_resource_conf(self, conf: dict[str, list[Any]]) -> CheckResult:
        if "cdn_managed_https" in conf and isinstance(conf["cdn_managed_https"], list):
            cdn = conf["cdn_managed_https"][0]
            if "tls_version" in cdn and isinstance(cdn["tls_version"], list) and cdn["tls_version"][0] in INSECURE_TLS_VERSIONS:
                return CheckResult.FAILED
        if "user_managed_https" in conf and isinstance(conf["user_managed_https"], list):
            user = conf["user_managed_https"][0]
            if "tls_version" in user and isinstance(user["tls_version"], list) and user["tls_version"][0] in INSECURE_TLS_VERSIONS:
                return CheckResult.FAILED
        return CheckResult.PASSED


check = CDNTLSProtocol12()



from __future__ import annotations

import logging
import os
from abc import abstractmethod
from collections.abc import Iterable
from pathlib import Path
from typing import cast, Any, TYPE_CHECKING, Generic, TypeVar

import docker

from checkov.common.bridgecrew.vulnerability_scanning.image_scanner import image_scanner
from checkov.common.bridgecrew.vulnerability_scanning.integrations.docker_image_scanning import \
    docker_image_scanning_integration
from checkov.common.output.common import ImageDetails
from checkov.common.output.report import Report, CheckType
from checkov.common.runners.base_runner import strtobool
from checkov.common.sca.commons import should_run_scan
from checkov.common.sca.output import add_to_report_sca_data, get_license_statuses

if TYPE_CHECKING:
    from checkov.common.bridgecrew.platform_integration import BcPlatformIntegration
    from checkov.runner_filter import RunnerFilter
    from networkx import DiGraph

_Definitions = TypeVar("_Definitions")


def fix_related_resource_ids(report: Report | None, tmp_dir: str) -> None:
    """Remove tmp dir prefix from 'relatedResourceId'"""

    if report and report.image_cached_results:
        for cached_result in report.image_cached_results:
            related_resource_id = cached_result.get("relatedResourceId")
            if related_resource_id and isinstance(related_resource_id, str):
                cached_result["relatedResourceId"] = related_resource_id.replace(tmp_dir, "", 1)


class Image:
    def __init__(self, file_path: str, name: str, start_line: int, end_line: int,
                 related_resource_id: str | None = None) -> None:
        """

        :param file_path: example: 'checkov/integration_tests/example_workflow_file/.github/workflows/vulnerable_container.yaml'
        :param name: example: 'node:14.16'
        :param image_id: example: 'sha256:6a353e22ce'
        :param start_line: example: 8
        :param end_line: example: 16
        """
        self.end_line = end_line
        self.start_line = start_line
        self.name = name
        self.file_path = file_path
        self.related_resource_id = related_resource_id

    def __eq__(self, other: Any) -> bool:
        if isinstance(other, self.__class__):
            return self.__dict__ == other.__dict__

        return False

    def __ne__(self, other: Any) -> bool:
        return not self.__eq__(other)

    def __hash__(self) -> int:
        return hash((self.file_path, self.name, self.start_line, self.end_line))


class ImageReferencer:
    @abstractmethod
    def is_workflow_file(self, file_path: str) -> bool:
        """

        :param file_path: path of file to validate if it is a file that contains might images (example: CI workflow file)
        :return: True if contains images

        """
        return False

    @abstractmethod
    def get_images(self, file_path: str) -> Iterable[Image]:
        """
        Get container images mentioned in a file
        :param file_path: File to be inspected
        :return: List of container images objects mentioned in the file.
        """
        return []

    @staticmethod
    def inspect(image_name: str) -> str:
        """

        :param image_name: name of the image to be inspected locally using a "docker inspect X". If image does not exist try to pull it locally.
        :return: short image id sha that is inspected. In case inspect has failed None will be returned.
        """
        try:
            logging.info("Inspecting docker image {}".format(image_name))
            client = docker.from_env()
            try:
                image = client.images.get(image_name)
            except Exception:
                image = client.images.pull(image_name)
                return cast(str, image.short_id)
            return cast(str, image.short_id)
        except Exception:
            logging.debug(f"failed to pull docker image={image_name}", exc_info=True)
            return ""


class ImageReferencerMixin(Generic[_Definitions]):
    """Mixin class to simplify image reference search"""

    def check_container_image_references(
        self,
        root_path: str | Path | None,
        runner_filter: RunnerFilter,
        graph_connector: DiGraph | None = None,
        definitions: _Definitions | None = None,
        definitions_raw: dict[str, list[tuple[int, str]]] | None = None,
    ) -> Report | None:
        """Tries to find image references in graph based IaC templates"""
        from checkov.common.bridgecrew.platform_integration import bc_integration

        # skip complete run, if flag '--check' was used without a CVE check ID
        if not should_run_scan(runner_filter.checks):
            return None

        images = self.extract_images(graph_connector=graph_connector, definitions=definitions,
                                     definitions_raw=definitions_raw)
        if not images:
            return None

        logging.info(f"Found {len(images)} image references {[image.name for image in images]}")

        report = Report(CheckType.SCA_IMAGE)
        root_path = Path(root_path) if root_path else None
        check_class = f"{image_scanner.__module__}.{image_scanner.__class__.__qualname__}"
        report_type = CheckType.SCA_IMAGE

        for image in images:
            self.add_image_records(
                report=report,
                root_path=root_path,
                check_class=check_class,
                dockerfile_path=image.file_path,
                image=image,
                runner_filter=runner_filter,
                report_type=report_type,
                bc_integration=bc_integration,
            )

        return report

    def add_image_records(
        self,
        report: Report,
        root_path: Path | None,
        check_class: str,
        dockerfile_path: str,
        image: Image,
        runner_filter: RunnerFilter,
        report_type: str,
        bc_integration: BcPlatformIntegration,
    ) -> None:
        """Adds an image record to the given report, if possible"""

        cached_results: dict[str, Any] | None = image_scanner.get_scan_results_from_cache(f"image:{image.name}")
        if cached_results:
            logging.info(f"Found cached scan results of image {image.name}")
            image_scanning_report: dict[str, Any] = docker_image_scanning_integration.create_report(
                twistcli_scan_result=cached_results,
                bc_platform_integration=bc_integration,
                file_path=dockerfile_path,
                file_content=f'image: {image.name}',
                docker_image_name=image.name,
                related_resource_id=image.related_resource_id,
                root_folder=root_path)
            report.image_cached_results.append(image_scanning_report)

            result = cached_results.get("results", [{}])[0]
            image_id = self.extract_image_short_id(result)
            image_details = self.get_image_details_from_twistcli_result(scan_result=result, image_id=image_id)
            if root_path:
                try:
                    dockerfile_path = str(Path(dockerfile_path).relative_to(root_path))
                except ValueError:
                    # Path.is_relative_to() was implemented in Python 3.9
                    pass
            rootless_file_path = dockerfile_path.replace(Path(dockerfile_path).anchor, "", 1)
            rootless_file_path_to_report = f"{rootless_file_path} ({image.name} lines:{image.start_line}-" \
                                           f"{image.end_line} ({image_id}))"

            self.add_vulnerability_records(
                report=report,
                result=result,
                check_class=check_class,
                dockerfile_path=dockerfile_path,
                rootless_file_path=rootless_file_path_to_report,
                image_details=image_details,
                runner_filter=runner_filter,
                report_type=report_type,
            )
        elif strtobool(os.getenv("CHECKOV_EXPERIMENTAL_IMAGE_REFERENCING", "False")):
            # experimental flag on running image referencers via local twistcli
            from checkov.sca_image.runner import Runner as sca_image_runner

            runner = sca_image_runner()

            image_id = ImageReferencer.inspect(image.name)
            if not image_id:
                return None

            scan_result = runner.scan(image_id, dockerfile_path, runner_filter)
            if scan_result is None:
                return None

            self.raw_report = scan_result
            result = scan_result.get('results', [{}])[0]
            rootless_file_path_to_report = f"{dockerfile_path} ({image.name} lines:{image.start_line}-" \
                                           f"{image.end_line} ({image_id}))"

            self.add_vulnerability_records(
                report=report,
                result=result,
                check_class=check_class,
                dockerfile_path=dockerfile_path,
                rootless_file_path=rootless_file_path_to_report,
                image_details=None,
                runner_filter=runner_filter,
                report_type=report_type,
            )
        else:
            logging.info(f"No cache hit for image {image.name}")

    def extract_image_short_id(self, scan_result: dict[str, Any]) -> str:
        """Extracts a shortened version of the image ID from the scan result"""

        if "id" not in scan_result:
            return "sha256:unknown"

        image_id: str = scan_result["id"]

        if image_id.startswith("sha256:"):
            return image_id[:17]
        return image_id[:10]

    def get_image_details_from_twistcli_result(self, scan_result: dict[str, Any], image_id: str) -> ImageDetails:
        """Extracts the image detaisl from a twistcli scan result"""

        image_packages = scan_result.get("packages", [])
        image_package_types = {f'{package["name"]}@{package["version"]}': package["type"] for package in image_packages}
        return ImageDetails(
            distro=scan_result.get("distro", ""),
            distro_release=scan_result.get("distroRelease", ""),
            package_types=image_package_types,
            image_id=image_id,
        )

    def add_vulnerability_records(
        self,
        report: Report,
        result: dict[str, Any],
        check_class: str,
        dockerfile_path: str,
        rootless_file_path: str,
        image_details: ImageDetails | None,
        runner_filter: RunnerFilter,
        report_type: str,
    ) -> None:
        vulnerabilities = result.get("vulnerabilities", [])
        packages = result.get("packages", [])
        license_statuses = get_license_statuses(packages)
        add_to_report_sca_data(
            report=report,
            check_class=check_class,
            scanned_file_path=os.path.abspath(dockerfile_path),
            rootless_file_path=rootless_file_path,
            runner_filter=runner_filter,
            vulnerabilities=vulnerabilities,
            packages=packages,
            license_statuses=license_statuses,
            sca_details=image_details,
            report_type=report_type,
        )

    @abstractmethod
    def extract_images(
        self,
        graph_connector: DiGraph | None = None,
        definitions: _Definitions | None = None,
        definitions_raw: dict[str, list[tuple[int, str]]] | None = None
    ) -> list[Image]:
        """Tries to find image references in the graph or supported resource"""

        pass



from __future__ import annotations

from typing import Any
from checkov.common.models.enums import CheckResult, CheckCategories
from checkov.common.checks.enums import BlockType
from checkov.openapi.checks.resource.v2.BaseOpenapiCheckV2 import BaseOpenapiCheckV2


class OperationObjectSecurityScopeUndefined(BaseOpenapiCheckV2):
    def __init__(self) -> None:
        id = "CKV_OPENAPI_9"
        name = "Ensure that security scopes of operations are defined in securityDefinitions - version 2.0 files"
        categories = [CheckCategories.API_SECURITY]
        supported_resources = ["paths"]
        super().__init__(
            name=name,
            id=id,
            categories=categories,
            supported_entities=supported_resources,
            block_type=BlockType.DOCUMENT,
        )

    def scan_openapi_conf(self, conf: dict[str, Any], entity_type: str) -> tuple[CheckResult, dict[str, Any]]:
        paths = conf.get("paths") or {}
        security_definitions = conf.get('securityDefinitions') or {}
        if not isinstance(paths, dict):
            return CheckResult.UNKNOWN, conf

        for path, path_dict in paths.items():
            if self.is_start_end_line(path):
                continue
            if not isinstance(path_dict, dict):
                return CheckResult.UNKNOWN, conf
            for operation, operation_dict in path_dict.items():
                if self.is_start_end_line(operation):
                    continue
                if not isinstance(operation_dict, dict):
                    return CheckResult.UNKNOWN, conf
                op_security = operation_dict.get('security', [{}])
                for security in op_security:
                    if not isinstance(security, dict):
                        return CheckResult.UNKNOWN, conf
                    for auth_key, auth_scopes in security.items():
                        if self.is_start_end_line(auth_key):
                            continue
                        if not isinstance(security_definitions, dict):
                            return CheckResult.UNKNOWN, conf
                        auth_definition = security_definitions.get(auth_key, {})
                        if not auth_definition:
                            return CheckResult.FAILED, conf
                        if not isinstance(auth_definition, dict):
                            return CheckResult.UNKNOWN, conf
                        definition_scopes = auth_definition.get('scopes', {})
                        if not definition_scopes:
                            return CheckResult.FAILED, conf
                        if not isinstance(auth_scopes, list):
                            return CheckResult.UNKNOWN, conf
                        for scope in auth_scopes:
                            if scope not in definition_scopes:
                                return CheckResult.FAILED, conf

        return CheckResult.PASSED, conf


check = OperationObjectSecurityScopeUndefined()



from __future__ import annotations

from typing import Any

from checkov.common.models.consts import ANY_VALUE
from checkov.common.models.enums import CheckCategories, CheckResult
from checkov.terraform.checks.resource.base_resource_value_check import BaseResourceValueCheck


class AKSApiServerAuthorizedIpRanges(BaseResourceValueCheck):
    def __init__(self) -> None:
        name = "Ensure AKS has an API Server Authorized IP Ranges enabled"
        id = "CKV_AZURE_6"
        supported_resources = ("azurerm_kubernetes_cluster",)
        categories = (CheckCategories.KUBERNETES,)
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def get_inspected_key(self) -> str:
        return "api_server_authorized_ip_ranges/[0]"

    def get_expected_value(self) -> Any:
        return ANY_VALUE

    def scan_resource_conf(self, conf: dict[str, list[Any]]) -> CheckResult:
        # can't be set for private cluster
        private_cluster_enabled = conf.get("private_cluster_enabled", [False])[0]
        if private_cluster_enabled:
            return CheckResult.PASSED
        return super().scan_resource_conf(conf)


check = AKSApiServerAuthorizedIpRanges()



from checkov.terraform.checks.resource.base_resource_value_check import BaseResourceValueCheck
from checkov.common.models.enums import CheckCategories


class DAXEndpointTLS(BaseResourceValueCheck):
    def __init__(self) -> None:
        name = "Ensure DAX cluster endpoint is using TLS"
        id = "CKV_AWS_239"
        supported_resources = ["aws_dax_cluster"]
        categories = [CheckCategories.ENCRYPTION]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def get_inspected_key(self) -> str:
        return "cluster_endpoint_encryption_type"

    def get_expected_value(self) -> str:
        return "TLS"


check = DAXEndpointTLS()



from checkov.common.models.enums import CheckCategories, CheckResult
from checkov.terraform.checks.resource.base_resource_value_check import BaseResourceValueCheck


class AppServiceAuthentication(BaseResourceValueCheck):
    def __init__(self):
        name = "Ensure App Service Authentication is set on Azure App Service"
        id = "CKV_AZURE_13"
        supported_resources = ('azurerm_app_service', 'azurerm_linux_web_app', 'azurerm_windows_web_app')
        categories = (CheckCategories.GENERAL_SECURITY,)
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources,
                         missing_block_result=CheckResult.FAILED)

    def get_inspected_key(self):
        return 'auth_settings/[0]/enabled/[0]'


check = AppServiceAuthentication()



from __future__ import annotations

from checkov.kubernetes.graph_builder.graph_components.edge_builders.K8SEdgeBuilder import K8SEdgeBuilder
from checkov.kubernetes.graph_builder.graph_components.blocks import KubernetesBlock
from checkov.kubernetes.kubernetes_utils import remove_metadata_from_attribute


class NetworkPolicyEdgeBuilder(K8SEdgeBuilder):

    @staticmethod
    def should_search_for_edges(vertex: KubernetesBlock) -> bool:
        return bool(vertex.attributes.get("kind") == "NetworkPolicy")

    @staticmethod
    def find_connections(vertex: KubernetesBlock, vertices: list[KubernetesBlock]) -> list[int]:
        """
        this edge builder is a specific case of LabelSelectorEdgeBuilder with 2 differences:
        1. it applies only to NetworkPolicy resources that connect to Pod resources
        2. it handles a wildcard that attaches a NetworkPolicy resource to all pods. for example:

        apiVersion: networking.k8s.io/v1
        kind: NetworkPolicy
        metadata:
          name: default-deny-ingress
        spec:
          podSelector: {}
          policyTypes:
          - Ingress
        """

        connections: list[int] = []
        for potential_pod_index, potential_vertex in enumerate(vertices):
            if potential_vertex.id == vertex.id or potential_vertex.attributes.get("kind") != "Pod":
                continue

            network_policy = vertex
            pod = potential_vertex

            pod_selector = network_policy.attributes.get("spec", {}).get("podSelector")
            if not pod_selector:
                continue
            match_labels = pod_selector.get("matchLabels")
            remove_metadata_from_attribute(match_labels)

            # the network policy has specific pod labels
            if match_labels and pod.metadata is not None and pod.metadata.labels is not None:
                pod_labels = pod.metadata.labels
                if len(match_labels) > len(pod_labels):
                    continue
                # find shared label between the inspected vertex and the iterated potential vertex
                shared_labels = [k for k in match_labels if k in pod_labels and match_labels[k] == pod_labels[k]]
                if len(shared_labels) == len(match_labels):
                    connections.append(potential_pod_index)
            # the network policy has a podSelector property with no labels and should apply for all pods
            else:
                connections.append(potential_pod_index)

        return connections



from __future__ import annotations

import os
from io import StringIO
from typing import Any, TYPE_CHECKING, cast

import configargparse

from checkov.common.bridgecrew.check_type import checkov_runners
from checkov.common.runners.runner_registry import OUTPUT_CHOICES, SUMMARY_POSITIONS
from checkov.common.util.consts import DEFAULT_EXTERNAL_MODULES_DIR
from checkov.common.util.type_forcers import convert_str_to_bool
from checkov.version import version

if TYPE_CHECKING:
    import argparse


class ExtArgumentParser(configargparse.ArgumentParser):
    def __init__(self, *args: Any, **kwargs: Any) -> None:
        super().__init__(*args, **kwargs)
        self.fields_to_sanitize: set[Any] = set()

    def add(self, *args: Any, **kwargs: Any) -> None:
        if kwargs.pop("sanitize", False):
            self.fields_to_sanitize.add(args[0])
        super().add(*args, **kwargs)

    def format_values(self, sanitize: bool = False) -> str:
        if not sanitize:
            return cast(str, super().format_values())

        source_key_to_display_value_map = {
            configargparse._COMMAND_LINE_SOURCE_KEY: "Command Line Args: ",
            configargparse._ENV_VAR_SOURCE_KEY: "Environment Variables:\n",
            configargparse._CONFIG_FILE_SOURCE_KEY: "Config File (%s):\n",
            configargparse._DEFAULTS_SOURCE_KEY: "Defaults:\n",
        }

        r = StringIO()
        for source, settings in self._source_to_settings.items():
            source = source.split("|")
            source = source_key_to_display_value_map[source[0]] % tuple(source[1:])
            r.write(source)
            for key, (action, value) in settings.items():
                if key:
                    if key in self.fields_to_sanitize or action.option_strings[0] in self.fields_to_sanitize:
                        value = "****"
                    r.write("  {:<19}{}\n".format(key + ":", value))
                else:
                    if isinstance(value, str):
                        r.write("  %s\n" % value)
                    elif isinstance(value, list):
                        value = list(value)  # copy
                        if source == "Command Line Args: ":
                            index = 0
                            while index < len(value):
                                if value[index] in self.fields_to_sanitize:
                                    index += 1
                                    value[index] = "****"
                                index += 1
                        r.write("  %s\n" % " ".join(value))

        return r.getvalue()

    def write_config_file(
        self, parsed_namespace: argparse.Namespace, output_file_paths: list[str], exit_after: bool = False
    ) -> None:
        """
        Write the given settings to output files. Overrides write_config_file from the class ArgumentParser for
        correcting types of some attributes (example: check, skip_check)

        :param parsed_namespace: namespace object created within parse_known_args()
        :param output_file_paths: any number of file paths to write the config to
        :param exit_after: whether to exit the program after writing the config files
        """
        for output_file_path in output_file_paths:
            # validate the output file path
            try:
                with self._config_file_open_func(output_file_path, "w") as output_file:
                    pass
            except IOError as e:
                raise ValueError(f"Couldn't open {output_file_path} for writing") from e
        if output_file_paths:
            # generate the config file contents
            config_items = self.get_items_for_config_file_output(self._source_to_settings, parsed_namespace)
            # convert check, skip_check, soft_fail_on and hard_fail_on to list
            if "check" in config_items.keys():
                config_items["check"] = config_items["check"][0].split(",")
            if "skip-check" in config_items.keys():
                config_items["skip-check"] = config_items["skip-check"][0].split(",")
            if "soft-fail-on" in config_items.keys():
                config_items["soft-fail-on"] = config_items["soft-fail-on"][0].split(",")
            if "hard-fail-on" in config_items.keys():
                config_items["hard-fail-on"] = config_items["hard-fail-on"][0].split(",")
            # convert strings to booleans
            for k in config_items.keys():
                config_items[k] = convert_str_to_bool(config_items[k])

            file_contents = self._config_file_parser.serialize(config_items)
            for output_file_path in output_file_paths:
                with self._config_file_open_func(output_file_path, "w") as output_file:
                    output_file.write(file_contents)
            message = "Wrote config file to " + ", ".join(output_file_paths)
            if exit_after:
                self.exit(0, message)
            else:
                print(message)

    def add_parser_args(self) -> None:
        self.add(
            "-v",
            "--version",
            help="version",
            action="version",
            version=version,
        )
        self.add(
            "-d",
            "--directory",
            action="append",
            help="IaC root directory (can not be used together with --file).",
        )
        self.add(
            "--add-check",
            action="store_true",
            help="Generate a new check via CLI prompt",
        )
        self.add(
            "-f",
            "--file",
            action="append",
            help="File to scan (can not be used together with --directory). With this option, Checkov will attempt "
            'to filter the runners based on the file type. For example, if you specify a ".tf" file, only the '
            "terraform and secrets frameworks will be included. You can further limit this (e.g., skip secrets) "
            "by using the --skip-framework argument.",
        )
        self.add(
            "--skip-path",
            action="append",
            help="Path (file or directory) to skip, using regular expression logic, relative to current "
            'working directory. Word boundaries are not implicit; i.e., specifying "dir1" will skip any '
            'directory or subdirectory named "dir1". Ignored with -f. Can be specified multiple times.',
        )
        self.add(
            "--external-checks-dir",
            action="append",
            help="Directory for custom checks to be loaded. Can be repeated",
        )
        self.add(
            "--external-checks-git",
            action="append",
            help="Github url of external checks to be added. \n you can specify a subdirectory after a "
            "double-slash //. \n cannot be used together with --external-checks-dir",
        )
        self.add(
            "-l",
            "--list",
            help="List checks",
            action="store_true",
        )
        self.add(
            "-o",
            "--output",
            action="append",
            choices=OUTPUT_CHOICES,
            default=None,
            help="Report output format. Add multiple outputs by using the flag multiple times (-o sarif -o cli)",
        )
        self.add(
            "--output-file-path",
            default=None,
            help="Name of the output folder to save the chosen output formats. "
            "Advanced usage: "
            "By using -o cli -o junitxml --output-file-path console,results.xml the CLI output will be printed "
            "to the console and the JunitXML output to the file results.xml.",
        )
        self.add(
            "--output-bc-ids",
            action="store_true",
            help="Print Bridgecrew platform IDs (BC...) instead of Checkov IDs (CKV...), if the check exists in the platform",
        )
        self.add(
            "--include-all-checkov-policies",
            action="store_true",
            help="When running with an API key, Checkov will omit any policies that do not exist "
            "in the Bridgecrew or Prisma Cloud platform, except for local custom policies loaded with the "
            "--external-check flags. Use this key to include policies that only exist in Checkov in the scan. "
            "Note that this will make the local CLI results different from the results you see in the "
            "platform. Has no effect if you are not using an API key. Use the --check option to explicitly "
            "include checks by ID even if they are not in the platform, without using this flag.",
        )
        self.add(
            "--quiet",
            action="store_true",
            default=False,
            help="in case of CLI output, display only failed checks. Also disables progress bars",
        )
        self.add(
            "--compact",
            action="store_true",
            default=False,
            help="in case of CLI output, do not display code blocks",
        )
        self.add(
            "--framework",
            help="Filter scan to run only on specific infrastructure code frameworks",
            choices=checkov_runners + ["all"],
            default=["all"],
            env_var="CKV_FRAMEWORK",
            nargs="+",
        )
        self.add(
            "--skip-framework",
            help="Filter scan to skip specific infrastructure as code frameworks."
            "This will be included automatically for some frameworks if system dependencies "
            "are missing. Add multiple frameworks using spaces. For example, "
            "--skip-framework terraform sca_package.",
            choices=checkov_runners,
            default=None,
            nargs="+",
        )
        self.add(
            "-c",
            "--check",
            help="Checks to run; any other checks will be skipped. Enter one or more items separated by commas. "
            "Each item may be either a Checkov check ID (CKV_AWS_123), a BC check ID (BC_AWS_GENERAL_123), or "
            "a severity (LOW, MEDIUM, HIGH, CRITICAL). If you use a severity, then all checks equal to or "
            "above the lowest severity in the list will be included. This option can be combined with "
            "--skip-check. If it is, then the logic is to first take all checks that match this list, and then "
            "remove all checks that match the skip list. For example, if you use --check CKV_123 and "
            "--skip-check LOW, then CKV_123 will not run if it is a LOW severity. Similarly, if you use "
            "--check CKV_789 --skip-check MEDIUM, then CKV_789 will run if it is a HIGH severity. If you use a "
            "check ID here along with an API key, and the check is not part of the BC / PC platform, then the "
            "check will still be run (see --include-all-checkov-policies for more info).",
            action="append",
            default=None,
            env_var="CKV_CHECK",
        )
        self.add(
            "--skip-check",
            help="Checks to skip; any other checks will not be run. Enter one or more items separated by commas. "
            "Each item may be either a Checkov check ID (CKV_AWS_123), a BC check ID (BC_AWS_GENERAL_123), or "
            "a severity (LOW, MEDIUM, HIGH, CRITICAL). If you use a severity, then all checks equal to or "
            "below the highest severity in the list will be skipped. This option can be combined with --check. "
            "If it is, priority is given to checks explicitly listed by ID or wildcard over checks listed by "
            "severity. For example, if you use --skip-check CKV_123 and --check HIGH, then CKV_123 will be "
            "skipped even if it is a HIGH severity. In the case of a tie (e.g., --check MEDIUM and "
            "--skip-check HIGH for a medium severity check), then the check will be skipped.",
            action="append",
            default=None,
            env_var="CKV_SKIP_CHECK",
        )
        self.add(
            "--run-all-external-checks",
            action="store_true",
            help="Run all external checks (loaded via --external-checks options) even if the checks are not present "
            "in the --check list. This allows you to always ensure that new checks present in the external "
            "source are used. If an external check is included in --skip-check, it will still be skipped.",
        )
        self.add(
            "-s",
            "--soft-fail",
            help="Runs checks but always returns a 0 exit code. Using either --soft-fail-on and / or --hard-fail-on "
            "overrides this option, except for the case when a result does not match either of the soft fail "
            "or hard fail criteria, in which case this flag determines the result.",
            action="store_true",
        )
        self.add(
            "--soft-fail-on",
            help="Exits with a 0 exit code if only the specified items fail. Enter one or more items "
            "separated by commas. Each item may be either a Checkov check ID (CKV_AWS_123), a BC "
            "check ID (BC_AWS_GENERAL_123), or a severity (LOW, MEDIUM, HIGH, CRITICAL). If you use "
            "a severity, then any severity equal to or less than the highest severity in the list "
            "will result in a soft fail. This option may be used with --hard-fail-on, using the same "
            "priority logic described in --check and --skip-check options above, with --hard-fail-on "
            "taking precedence in a tie. If a given result does not meet the --soft-fail-on nor "
            "the --hard-fail-on criteria, then the default is to hard fail",
            action="append",
            default=None,
        )
        self.add(
            "--hard-fail-on",
            help="Exits with a non-zero exit code for specified checks. Enter one or more items "
            "separated by commas. Each item may be either a Checkov check ID (CKV_AWS_123), a BC "
            "check ID (BC_AWS_GENERAL_123), or a severity (LOW, MEDIUM, HIGH, CRITICAL). If you use a "
            "severity, then any severity equal to or greater than the lowest severity in the list will "
            "result in a hard fail. This option can be used with --soft-fail-on, using the same "
            "priority logic described in --check and --skip-check options above, with --hard-fail-on "
            "taking precedence in a tie.",
            action="append",
            default=None,
        )
        self.add(
            "--bc-api-key",
            env_var="BC_API_KEY",
            sanitize=True,
            help="Bridgecrew API key or Prisma Cloud Access Key (see --prisma-api-url)",
        )
        self.add(
            "--prisma-api-url",
            env_var="PRISMA_API_URL",
            default=None,
            help="The Prisma Cloud API URL (see: https://prisma.pan.dev/api/cloud/api-urls). "
            "Requires --bc-api-key to be a Prisma Cloud Access Key in the following format: <access_key_id>::<secret_key>",
        )
        self.add(
            "--docker-image",
            help="Scan docker images by name or ID. Only works with --bc-api-key flag",
        )
        self.add(
            "--dockerfile-path",
            help="Path to the Dockerfile of the scanned docker image",
        )
        self.add(
            "--repo-id",
            help="Identity string of the repository, with form <repo_owner>/<repo_name>",
        )
        self.add(
            "-b",
            "--branch",
            help="Selected branch of the persisted repository. Only has effect when using the --bc-api-key flag",
            default="master",
        )
        self.add(
            "--skip-download",
            help="Do not download any data from Bridgecrew. This will omit doc links, severities, etc., as well as "
            "custom policies and suppressions if using an API token. Note: it will prevent BC platform IDs from "
            "being available in Checkov.",
            action="store_true",
        )
        self.add(
            "--use-enforcement-rules",
            action="store_true",
            help="Use the Enforcement rules configured in the platform for hard / soft fail logic. With this option, "
            "the enforcement rule matching this repo, or the default rule if there is no match, will determine "
            "this behavior: any check with a severity below the selected rule's soft-fail threshold will be "
            "skipped; any check with a severity equal to or greater than the rule's hard-fail threshold will "
            "be part of the hard-fail list, and any check in between will be part of the soft-fail list. For "
            "example, if the given enforcement rule has a hard-fail value of HIGH and a soft-fail value of MEDIUM,"
            "this is the equivalent of using the flags `--skip-check LOW --hard-fail-on HIGH`. You can use --check, "
            "--skip-check, --soft-fail, --soft-fail-on, or --hard-fail-on to override portions of an enforcement rule. "
            "Note, however, that the logic of applying the --check list and then the --skip-check list (as described "
            "above under --check) still applies here. Requires a BC or PC platform API key.",
        )
        self.add(
            "--no-guide",
            action="store_true",
            default=False,
            help="Deprecated - use --skip-download",
        )
        self.add(
            "--skip-suppressions",
            help="Deprecated - use --skip-download",
            action="store_true",
        )
        self.add(
            "--skip-policy-download",
            help="Deprecated - use --skip-download",
            action="store_true",
        )
        self.add(
            "--skip-fixes",
            help="Do not download fixed resource templates from Bridgecrew. Only has effect when using the API key.",
            action="store_true",
        )
        self.add(
            "--download-external-modules",
            help="download external terraform modules from public git repositories and terraform registry",
            default=os.getenv("DOWNLOAD_EXTERNAL_MODULES", False),
            env_var="DOWNLOAD_EXTERNAL_MODULES",
        )
        self.add(
            "--var-file",
            action="append",
            help="Variable files to load in addition to the default files (see "
            "https://www.terraform.io/docs/language/values/variables.html#variable-definitions-tfvars-files)."
            "Currently only supported for source Terraform (.tf file), and Helm chart scans."
            "Requires using --directory, not --file.",
        )
        self.add(
            "--external-modules-download-path",
            help="set the path for the download external terraform modules",
            default=DEFAULT_EXTERNAL_MODULES_DIR,
            env_var="EXTERNAL_MODULES_DIR",
        )
        self.add(
            "--evaluate-variables",
            help="evaluate the values of variables and locals",
            env_var="CKV_EVAL_VARS",
            default=True,
        )
        self.add(
            "-ca", "--ca-certificate", help="Custom CA certificate (bundle) file", default=None, env_var="BC_CA_BUNDLE"
        )
        self.add(
            "--repo-root-for-plan-enrichment",
            help="Directory containing the hcl code used to generate a given plan file. Use with -f.",
            dest="repo_root_for_plan_enrichment",
            action="append",
        )
        self.add(
            "--config-file",
            help="path to the Checkov configuration YAML file",
            is_config_file=True,
            default=None,
        )
        self.add(
            "--create-config",
            help="takes the current command line args and writes them out to a config file at " "the given path",
            is_write_out_config_file_arg=True,
            default=None,
        )
        self.add(
            "--show-config",
            help="prints all args and config settings and where they came from "
            "(eg. commandline, config file, environment variable or default)",
            action="store_true",
            default=None,
        )
        self.add(
            "--create-baseline",
            help="Alongside outputting the findings, save all results to .checkov.baseline file"
            " so future runs will not re-flag the same noise. Works only with `--directory` flag",
            action="store_true",
            default=False,
        )
        self.add(
            "--baseline",
            help="Use a .checkov.baseline file to compare current results with a known baseline. "
            "Report will include only failed checks that are new with respect to the provided baseline",
            default=None,
        )
        self.add(
            "--output-baseline-as-skipped",
            help="output checks that are skipped due to baseline file presence",
            action="store_true",
            default=False,
        )
        self.add(
            "--skip-cve-package",
            help="filter scan to run on all packages but a specific package identifier (denylist), You can "
            "specify this argument multiple times to skip multiple packages",
            action="append",
            default=None,
        )
        self.add(
            "--policy-metadata-filter",
            help="comma separated key:value string to filter policies based on Prisma Cloud policy metadata. "
            "See https://prisma.pan.dev/api/cloud/cspm/policy#operation/get-policy-filters-and-options for "
            "information on allowed filters. Format: policy.label=test,cloud.type=aws ",
            default=None,
        )
        self.add(
            "--secrets-scan-file-type",
            default=[],
            env_var="CKV_SECRETS_SCAN_FILE_TYPE",
            action="append",
            help="not in use",
        )
        self.add(
            "--enable-secret-scan-all-files",
            default=False,
            env_var="CKV_SECRETS_SCAN_ENABLE_ALL",
            action="store_true",
            help="enable secret scan for all files",
        )
        self.add(
            "--block-list-secret-scan",
            default=[],
            env_var="CKV_SECRETS_SCAN_BLOCK_LIST",
            action="append",
            help="List of files to filter out from the secret scanner",
        )
        self.add(
            "--summary-position",
            default="top",
            choices=SUMMARY_POSITIONS,
            help="Chose whether the summary will be appended on top (before the checks results) or on bottom "
            "(after check results), default is on top.",
        )
        self.add(
            "--skip-resources-without-violations",
            help="exclude extra resources (resources without violations) from report output",
            action="store_true",
            env_var="CKV_SKIP_RESOURCES_WITHOUT_VIOLATIONS",
        )
        self.add(
            "--deep-analysis",
            default=False,
            action="store_true",
            help="Enable combine tf graph and rf plan graph",
        )
        self.add(
            "--no-fail-on-crash",
            default=False,
            env_var="CKV_NO_FAIL_ON_CRASH",
            action="store_true",
            help="Return exit code 0 instead of 2",
        )
        self.add(
            "--mask",
            action="append",
            default=[],
            help="List of <resource_type>:<variable> OR <variable> only. Each entry in the list will be used for"
                 "masking the desired attribute for resource (or for all resources, if no resource given)."
                 "Notice: one entry can contain several variables, seperated with a comma. For example:"
                 "<resource_type>:<variable1>,<variable2> OR <variable1>,<variable2>"
        )



from checkov.terraform.checks.resource.base_resource_check import BaseResourceCheck
from checkov.common.models.enums import CheckResult, CheckCategories


class NACLInboundCheck(BaseResourceCheck):
    def __init__(self, check_id, port):
        name = f"Ensure no NACL allow inbound from 0.0.0.0:0 to port {port}"
        id = check_id
        supported_resources = ('ncloud_network_acl_rule',)
        categories = (CheckCategories.NETWORKING,)
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)
        self.port = port

    def scan_resource_conf(self, conf):
        for inbound in conf.get('inbound', []):
            if inbound['rule_action'] == ["ALLOW"]:
                ip = inbound.get('ip_block', ['0.0.0.0/0'])
                if ip == ['0.0.0.0/0'] or ip == ['::/0']:
                    port = inbound.get('port_range', str(self.port))[0]
                    if port == str(self.port):
                        return CheckResult.FAILED
                    elif port.find('-'):
                        port_range = list(map(int, port.split("-")))
                        if port_range[0] <= self.port <= port_range[-1]:
                            return CheckResult.FAILED

        return CheckResult.PASSED



from checkov.common.models.enums import CheckResult, CheckCategories
from checkov.terraform.checks.resource.base_resource_value_check import BaseResourceCheck


class RouteTableNATGatewayDefault(BaseResourceCheck):
    def __init__(self):
        name = "Ensure Routing Table associated with Web tier subnet have the default route (0.0.0.0/0) defined to " \
               "allow connectivity "
        id = "CKV_NCP_20"
        supported_resources = ('ncloud_route',)
        categories = (CheckCategories.NETWORKING,)
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def scan_resource_conf(self, conf):
        if "destination_cidr_block" in conf.keys() and "target_type" in conf.keys():
            if conf.get("target_type") == ["NATGW"]:
                if conf.get("destination_cidr_block") == ["0.0.0.0/0"]:
                    return CheckResult.PASSED
                else:
                    return CheckResult.FAILED
        return CheckResult.UNKNOWN


check = RouteTableNATGatewayDefault()



from checkov.terraform.checks.resource.base_resource_value_check import BaseResourceValueCheck
from checkov.common.models.enums import CheckCategories


class CloudStorageVersioningEnabled(BaseResourceValueCheck):
    def __init__(self):
        name = "Ensure Cloud storage has versioning enabled"
        id = "CKV_GCP_78"
        supported_resources = ['google_storage_bucket']
        categories = [CheckCategories.LOGGING]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def get_inspected_key(self):
        return "versioning/[0]/enabled"


check = CloudStorageVersioningEnabled()



from typing import Any

from checkov.common.models.enums import CheckCategories
from checkov.terraform.checks.resource.base_resource_value_check import BaseResourceValueCheck


class AKSIsPaidSku(BaseResourceValueCheck):
    def __init__(self) -> None:
        name = "Ensure that AKS use the Paid Sku for its SLA"
        # AKS clusters should have Uptime
        # SLA enabled to ensure availability
        # of control plane components
        # for production workloads.
        id = "CKV_AZURE_170"
        supported_resources = ("azurerm_kubernetes_cluster",)
        categories = (CheckCategories.GENERAL_SECURITY,)
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def get_inspected_key(self) -> str:
        return "sku_tier"

    def get_expected_value(self) -> Any:
        return "Paid"


check = AKSIsPaidSku()



from __future__ import annotations

from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from checkov.common.bridgecrew.run_metadata.abstract_run_metadata_extractor import RunMetaDataExtractor


class RunMetadataExtractorsRegistry:
    def __init__(self) -> None:
        self.extractors: set[RunMetaDataExtractor] = set()

    def register(self, extractor: RunMetaDataExtractor) -> None:
        self.extractors.add(extractor)

    def get_extractor(self) -> RunMetaDataExtractor:
        for extractor in self.extractors:
            if extractor.is_current_ci():
                return extractor
        for extractor in self.extractors:
            if extractor.__class__.__name__ == "DefaultRunMetadataExtractor":
                return extractor

        # should never be reached
        from checkov.common.bridgecrew.run_metadata.extractors.default_extractor import DefaultRunMetadataExtractor
        return DefaultRunMetadataExtractor()


registry = RunMetadataExtractorsRegistry()



from __future__ import annotations

from typing import Any

from checkov.common.models.enums import CheckResult, CheckCategories
from checkov.arm.base_resource_check import BaseResourceCheck

# https://docs.microsoft.com/en-us/azure/templates/microsoft.web/2019-08-01/sites/config-authsettings


class AppServiceAuthentication(BaseResourceCheck):
    def __init__(self) -> None:
        name = "Ensure App Service Authentication is set on Azure App Service"
        id = "CKV_AZURE_13"
        supported_resources = ("Microsoft.Web/sites/config", "config")
        categories = (CheckCategories.IAM,)
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def scan_resource_conf(self, conf: dict[str, Any]) -> CheckResult:
        if self.entity_type == "Microsoft.Web/sites/config":
            if "name" in conf and "authsettings" in conf["name"]:
                if "properties" in conf and "enabled" in conf["properties"]:
                    if str(conf["properties"]["enabled"]).lower() == "true":
                        return CheckResult.PASSED
            return CheckResult.FAILED
        elif self.entity_type == "config":
            if "name" in conf and conf["name"] == "authsettings":
                if "parent_type" in conf:
                    if conf["parent_type"] == "Microsoft.Web/sites":
                        if "properties" in conf:
                            if "enabled" in conf["properties"]:
                                if str(conf["properties"]["enabled"]).lower() == "true":
                                    return CheckResult.PASSED
                    return CheckResult.FAILED

        return CheckResult.UNKNOWN


check = AppServiceAuthentication()



from __future__ import annotations

from typing import Any

from checkov.common.models.enums import CheckCategories, CheckResult
from checkov.terraform.checks.resource.base_resource_check import BaseResourceCheck


class RDSRetention(BaseResourceCheck):
    def __init__(self) -> None:
        """
        'sql_collector_status' should be defined and set to "Enabled"  (I know, really, who does that)
         and 'sql_collector_config_value' should be defined and set to 180 or more
        """
        name = "Ensure RDS Instance SQL Collector Retention Period should be greater than 180"
        id = "CKV_ALI_25"
        supported_resources = ("alicloud_db_instance",)
        categories = (CheckCategories.LOGGING,)
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def scan_resource_conf(self, conf: dict[str, list[Any]]) -> CheckResult:
        collector_status = conf.get("sql_collector_status")
        if collector_status and isinstance(collector_status, list):
            if collector_status[0] != "Enabled":
                self.evaluated_keys = ["sql_collector_status"]
                return CheckResult.FAILED
            collector_config = conf.get("sql_collector_config_value")
            if collector_config and isinstance(collector_config, list):
                self.evaluated_keys = ["sql_collector_config_value"]
                if collector_config[0] >= 180:
                    return CheckResult.PASSED

        return CheckResult.FAILED


check = RDSRetention()



from __future__ import annotations

from typing import Any

from checkov.arm.base_resource_check import BaseResourceCheck
from checkov.common.models.enums import CheckResult, CheckCategories
from checkov.common.util.type_forcers import force_int


class StorageAccountsTransportEncryption(BaseResourceCheck):
    def __init__(self) -> None:
        # supportsHttpsTrafficOnly: Allows https traffic only to storage service if sets to true. The default value is
        # true since API version 2019-04-01.
        name = "Ensure that 'supportsHttpsTrafficOnly' is set to 'true'"
        id = "CKV_AZURE_3"
        supported_resources = ("Microsoft.Storage/storageAccounts",)
        categories = (CheckCategories.ENCRYPTION,)
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def scan_resource_conf(self, conf: dict[str, Any]) -> CheckResult:
        properties = conf.get("properties")
        if isinstance(properties, dict) and "supportsHttpsTrafficOnly" in properties:
            https = str(properties["supportsHttpsTrafficOnly"]).lower()
            return CheckResult.PASSED if https == "true" else CheckResult.FAILED

        # Use default if supportsHttpsTrafficOnly is not set
        if "apiVersion" in conf:
            # Default for apiVersion 2019 and newer is supportsHttpsTrafficOnly = True
            year = force_int(conf["apiVersion"][0:4])

            if year is None:
                return CheckResult.UNKNOWN
            elif year < 2019:
                return CheckResult.FAILED
            else:
                return CheckResult.PASSED
        return CheckResult.FAILED


check = StorageAccountsTransportEncryption()



from checkov.terraform.checks.resource.aws.AbsSecurityGroupUnrestrictedIngress import AbsSecurityGroupUnrestrictedIngress


class SecurityGroupUnrestrictedIngress80(AbsSecurityGroupUnrestrictedIngress):
    def __init__(self):
        super().__init__(check_id="CKV_AWS_260", port=80)


check = SecurityGroupUnrestrictedIngress80()



from __future__ import annotations

import itertools
import json
import logging
from typing import Optional, Tuple, Dict, List, Any

from checkov.common.graph.graph_builder import CustomAttributes
from checkov.common.parsers.node import DictNode, ListNode
from checkov.terraform.context_parsers.tf_plan import parse

SIMPLE_TYPES = (str, int, float, bool)
TF_PLAN_RESOURCE_ADDRESS = "__address__"
TF_PLAN_RESOURCE_CHANGE_ACTIONS = "__change_actions__"

RESOURCE_TYPES_JSONIFY = {
    "aws_batch_job_definition": "container_properties",
    "aws_ecs_task_definition": "container_definitions",
    "aws_iam_policy": "policy",
    "aws_iam_role": "assume_role_policy",
    "aws_iam_role_policy": "policy",
    "aws_iam_group_policy": "policy",
    "aws_iam_user_policy": "policy",
    "aws_ssoadmin_permission_set_inline_policy": "inline_policy",
}


def _is_simple_type(obj: Any) -> bool:
    if obj is None:
        return True
    if isinstance(obj, SIMPLE_TYPES):
        return True
    return False


def _is_list_of_simple_types(obj: Any) -> bool:
    if not isinstance(obj, list):
        return False
    for i in obj:
        if not _is_simple_type(i):
            return False
    return True


def _is_list_of_dicts(obj: Any) -> bool:
    if not isinstance(obj, list):
        return False
    for i in obj:
        if isinstance(i, dict):
            return True
    return False


def _hclify(
    obj: dict[str, Any],
    conf: dict[str, Any] | None = None,
    parent_key: str | None = None,
    resource_type: str | None = None,
) -> dict[str, list[Any]]:
    ret_dict = {}

    if not isinstance(obj, dict):
        raise Exception("this method receives only dicts")

    if resource_type and resource_type in RESOURCE_TYPES_JSONIFY:
        jsonify(obj=obj, resource_type=resource_type)

    if hasattr(obj, "start_mark") and hasattr(obj, "end_mark"):
        obj["start_line"] = obj.start_mark.line
        obj["end_line"] = obj.end_mark.line
    for key, value in obj.items():
        if _is_simple_type(value) or _is_list_of_simple_types(value):
            if parent_key == "tags":
                ret_dict[key] = value
            else:
                ret_dict[key] = _clean_simple_type_list([value])

        if _is_list_of_dicts(value):
            child_list = []
            conf_val = conf.get(key, []) if conf else []
            if not isinstance(conf_val, list):
                # this occurs, when a resource in the current state has no value for that argument
                conf_val = [conf_val]

            for internal_val, internal_conf_val in itertools.zip_longest(value, conf_val):
                if isinstance(internal_val, dict):
                    child_list.append(_hclify(internal_val, internal_conf_val, parent_key=key))
            if key == "tags":
                ret_dict[key] = [child_list]
            else:
                ret_dict[key] = child_list
        if isinstance(value, dict):
            child_dict = _hclify(value, parent_key=key)
            if parent_key == "tags":
                ret_dict[key] = child_dict
            else:
                ret_dict[key] = [child_dict]
    if conf and isinstance(conf, dict):
        found_ref = False
        for conf_key in conf.keys() - obj.keys():
            conf_value = conf[conf_key]
            if not isinstance(conf_value, dict):
                continue

            ref = next((x for x in conf_value.get("references", []) if not x.startswith(("var.", "local."))), None)
            if ref:
                ret_dict[conf_key] = [ref]
                found_ref = True
        if not found_ref:
            ret_dict[CustomAttributes.REFERENCES] = [
                value["references"]
                for value in conf.values()
                if isinstance(value, dict) and "references" in value
            ]

    return ret_dict


def jsonify(obj: dict[str, Any], resource_type: str) -> None:
    """Tries to create a dict from a string of a supported resource type attribute"""

    jsonify_key = RESOURCE_TYPES_JSONIFY[resource_type]
    if jsonify_key in obj:
        try:
            obj[jsonify_key] = json.loads(obj[jsonify_key])
        except json.JSONDecodeError:
            logging.debug(
                f"Attribute {jsonify_key} of resource type {resource_type} is not json encoded {obj[jsonify_key]}"
            )


def _prepare_resource_block(
    resource: DictNode, conf: Optional[DictNode], resource_changes: dict[str, dict[str, Any]]
) -> tuple[dict[str, dict[str, Any]], bool]:
    """hclify resource if pre-conditions met.

    :param resource: tf planned_values resource block
    :param conf: tf configuration resource block
    :param resource_changes: tf resource_changes block

    :returns:
        - resource_block: a list of strings representing the header columns
        - prepared: whether conditions met to prepare data
    """

    resource_block: Dict[str, Dict[str, Any]] = {}
    resource_block[resource["type"]] = {}
    prepared = False
    mode = ""
    if "mode" in resource:
        mode = resource.get("mode")
    # Rare cases where data block appears in resources with same name as resource block and only partial values
    # and where *_module resources don't have values field
    if mode == "managed":
        expressions = conf.get("expressions") if conf else None

        resource_conf = _hclify(
            obj=resource.get("values", {"start_line": 0, "end_line": 0}),
            conf=expressions,
            resource_type=resource.get("type"),
        )
        resource_address = resource.get("address")
        resource_conf[TF_PLAN_RESOURCE_ADDRESS] = resource_address

        changes = resource_changes.get(resource_address)
        if changes:
            resource_conf[TF_PLAN_RESOURCE_CHANGE_ACTIONS] = changes.get("change", {}).get("actions") or []

        resource_block[resource["type"]][resource.get("name", "default")] = resource_conf
        prepared = True
    return resource_block, prepared


def _find_child_modules(
    child_modules: ListNode, resource_changes: dict[str, dict[str, Any]], root_module_conf: dict[str, Any]
) -> List[Dict[str, Dict[str, Any]]]:
    """ Find all child modules if any. Including any amount of nested child modules.

    :param child_modules: list of terraform child_module objects
    :param resource_changes: a resource address to resource changes dict
    :param root_module_conf: configuration block of the root module
    :returns:
        list of terraform resource blocks
    """

    resource_blocks = []
    for child_module in child_modules:
        nested_child_modules = child_module.get("child_modules", [])
        if nested_child_modules:
            nested_blocks = _find_child_modules(
                child_modules=nested_child_modules,
                resource_changes=resource_changes,
                root_module_conf=root_module_conf
            )
            for resource in nested_blocks:
                resource_blocks.append(resource)

        module_address = child_module.get("address", "")
        module_call_resources = _get_module_call_resources(
            module_address=module_address,
            root_module_conf=root_module_conf,
        )

        for resource in child_module.get("resources", []):
            module_call_conf = None
            if module_address and module_call_resources:
                module_call_conf = next(
                    (
                        module_call_resource
                        for module_call_resource in module_call_resources
                        if f"{module_address}.{module_call_resource['address']}" == resource["address"]
                    ),
                    None
                )

            resource_block, prepared = _prepare_resource_block(
                resource=resource,
                conf=module_call_conf,
                resource_changes=resource_changes,
            )
            if prepared is True:
                resource_blocks.append(resource_block)
    return resource_blocks


def _get_module_call_resources(module_address: str, root_module_conf: dict[str, Any]) -> list[dict[str, Any]]:
    """Extracts the resources from the 'module_calls' block under 'configuration'"""

    for module_name in module_address.split("."):
        if module_name == "module":
            # module names are always prefixed with 'module.', therefore skip it
            continue
        root_module_conf = root_module_conf.get("module_calls", {}).get(module_name, {}).get("module", {})

    return root_module_conf.get("resources", [])


def _get_resource_changes(template: dict[str, Any]) -> dict[str, dict[str, Any]]:
    """Returns a resource address to resource changes dict"""

    resource_changes_map = {}

    resource_changes = template.get("resource_changes")
    if resource_changes and isinstance(resource_changes, list):
        resource_changes_map = {
            change.get("address", ""): change
            for change in resource_changes
        }

    return resource_changes_map


def parse_tf_plan(tf_plan_file: str, out_parsing_errors: Dict[str, str]) -> Tuple[Optional[Dict[str, Any]], Optional[List[Tuple[int, str]]]]:
    """
    :type tf_plan_file: str - path to plan file
    :rtype: tf_definition dictionary and template_lines of the plan file
    """
    tf_definition: Dict[str, Any] = {"resource": []}
    template, template_lines = parse(tf_plan_file, out_parsing_errors)
    if not template:
        return None, None

    resource_changes = _get_resource_changes(template=template)

    for resource in template.get("planned_values", {}).get("root_module", {}).get("resources", []):
        conf = next(
            (
                x
                for x in template.get("configuration", {}).get("root_module", {}).get("resources", [])
                if x["type"] == resource["type"] and x["name"] == resource["name"]
            ),
            None,
        )
        resource_block, prepared = _prepare_resource_block(
            resource=resource,
            conf=conf,
            resource_changes=resource_changes,
        )
        if prepared is True:
            tf_definition["resource"].append(resource_block)
    child_modules = template.get("planned_values", {}).get("root_module", {}).get("child_modules", [])
    root_module_conf = template.get("configuration", {}).get("root_module", {})
    # Terraform supports modules within modules so we need to search
    # in nested modules to find all resource blocks
    resource_blocks = _find_child_modules(
        child_modules=child_modules,
        resource_changes=resource_changes,
        root_module_conf=root_module_conf,
    )
    for resource in resource_blocks:
        tf_definition["resource"].append(resource)
    return tf_definition, template_lines


def _clean_simple_type_list(value_list: List[Any]) -> List[Any]:
    """
    Given a list of simple types return a cleaned list of simple types.
    Converts booleans that are input as strings back to booleans to maintain consistent expectations for later evaluation.
    Sometimes Terraform Plan will output Map values as strings regardless of boolean input.
    """
    for i in range(len(value_list)):
        if isinstance(value_list[i], str):
            lower_case_value = value_list[i].lower()
            if lower_case_value == "true":
                value_list[i] = True
            if lower_case_value == "false":
                value_list[i] = False
    return value_list



from typing import Dict, List, Any, Optional

from checkov.common.util.type_forcers import force_dict


def get_resource_tags(entity_config: Dict[str, List[Any]]) -> Optional[Dict[str, Any]]:
    return force_dict(entity_config.get("tags"))



from __future__ import annotations

import logging
from copy import deepcopy
from typing import Any, List, Dict
from collections import defaultdict

from checkov.common.graph.graph_builder import Edge
from checkov.common.graph.graph_builder.local_graph import LocalGraph
from checkov.common.util.consts import START_LINE, END_LINE
from checkov.kubernetes.graph_builder.graph_components.blocks import KubernetesBlock, KubernetesBlockMetadata, KubernetesSelector
from checkov.kubernetes.kubernetes_utils import DEFAULT_NESTED_RESOURCE_TYPE, is_invalid_k8_definition, get_resource_id, is_invalid_k8_pod_definition, \
    remove_metadata_from_attribute, PARENT_RESOURCE_KEY_NAME, PARENT_RESOURCE_ID_KEY_NAME, SUPPORTED_POD_CONTAINERS_TYPES
from checkov.kubernetes.kubernetes_graph_flags import K8sGraphFlags
from checkov.kubernetes.graph_builder.graph_components.edge_builders.LabelSelectorEdgeBuilder import LabelSelectorEdgeBuilder
from checkov.kubernetes.graph_builder.graph_components.edge_builders.KeywordEdgeBuilder import KeywordEdgeBuilder
from checkov.kubernetes.graph_builder.graph_components.edge_builders.NetworkPolicyEdgeBuilder import NetworkPolicyEdgeBuilder


class KubernetesLocalGraph(LocalGraph[KubernetesBlock]):
    def __init__(self, definitions: dict[str, list[dict[str, Any]]]) -> None:
        self.definitions = definitions
        self.edge_builders = (LabelSelectorEdgeBuilder, KeywordEdgeBuilder, NetworkPolicyEdgeBuilder)
        super().__init__()

    def build_graph(self, render_variables: bool, graph_flags: K8sGraphFlags | None = None) -> None:
        if graph_flags is None:
            graph_flags = K8sGraphFlags()

        self._create_vertices(create_complex_vertices=graph_flags.create_complex_vertices)
        if graph_flags.create_edges:
            self._create_edges()

    def _create_vertices(self, create_complex_vertices: bool) -> None:
        for file_path, file_conf in self.definitions.items():
            for resource in file_conf:
                if resource.get('kind') == "List":
                    file_conf.extend(item for item in resource.get("items", []) if item)
                    file_conf.remove(resource)

            if create_complex_vertices:
                file_conf = self._extract_nested_resources(file_conf)

            for resource in file_conf:
                resource_type = resource.get('kind', DEFAULT_NESTED_RESOURCE_TYPE)
                metadata = resource.get('metadata') or {}
                # TODO: add support for generateName
                
                if resource_type == DEFAULT_NESTED_RESOURCE_TYPE:
                    if is_invalid_k8_pod_definition(resource):
                        logging.info(f"failed to create a vertex in file {file_path}")
                        file_conf.remove(resource)
                        continue
                    
                else:
                    if is_invalid_k8_definition(resource) or not metadata.get('name'):
                        logging.info(f"failed to create a vertex in file {file_path}")
                        file_conf.remove(resource)
                        continue

                config = deepcopy(resource)
                attributes = deepcopy(config)
                attributes["resource_type"] = resource_type
                attributes["kind"] = resource_type
                attributes[START_LINE] = resource[START_LINE]
                attributes[END_LINE] = resource[END_LINE]

                block_id = get_resource_id(resource)
                if not block_id:
                    continue

                block_metadata = None
                if create_complex_vertices:
                    block_metadata = KubernetesLocalGraph._get_k8s_block_metadata(resource)

                self.vertices.append(KubernetesBlock(
                    block_name=block_id,
                    resource_type=resource_type,
                    config=config,
                    path=file_path,
                    attributes=attributes,
                    metadata=block_metadata
                ))

        for i, vertex in enumerate(self.vertices):
            self.vertices_by_block_type[vertex.block_type].append(i)
            self.vertices_block_name_map[vertex.block_type][vertex.name].append(i)

    def _create_edges(self) -> None:
        edges_to_create = defaultdict(list)
        for vertex_index, vertex in enumerate(self.vertices):
            for edge_builder in self.edge_builders:
                if edge_builder.should_search_for_edges(vertex):
                    current_vertex_connections = edge_builder.find_connections(vertex, self.vertices)
                    if current_vertex_connections:
                        edges_to_create[vertex.name].extend(current_vertex_connections)
            for destination_vertex_index in edges_to_create[vertex.name]:
                self._create_edge(vertex_index, destination_vertex_index, vertex.name)

    def _create_edge(self, origin_vertex_index: int, dest_vertex_index: int, label: str) -> None:
        edge = Edge(origin_vertex_index, dest_vertex_index, label)
        self.edges.append(edge)
        self.out_edges[origin_vertex_index].append(edge)
        self.in_edges[dest_vertex_index].append(edge)

    @staticmethod
    def _get_k8s_block_metadata(resource: Dict[str, Any]) -> KubernetesBlockMetadata:
        name = resource.get('metadata', {}).get('name')
        spec = resource.get('spec')
        match_labels: Dict[str, Any] | None = None
        if isinstance(spec, list):
            for spec_item in spec:
                if spec_item.get('selector'):
                    if resource.get('kind') == "Service":
                        match_labels = spec_item.get('selector')
                    else:
                        match_labels = spec_item.get('selector').get('matchLabels')
                    break
            else:
                match_labels = None
        elif isinstance(spec, dict):
            if spec.get('selector'):
                if resource.get('kind') == "Service":
                    match_labels = spec.get('selector')
                else:
                    match_labels = spec.get('selector', {}).get('matchLabels')
        remove_metadata_from_attribute(match_labels)
        selector = KubernetesSelector(match_labels)
        labels = resource.get('metadata', {}).get('labels')
        remove_metadata_from_attribute(labels)
        return KubernetesBlockMetadata(selector, labels, name)

    @staticmethod
    def _extract_nested_resources(file_conf: List[Dict[str, Any]]) -> List[Dict[str, Any]]:
        all_resources: "list[dict[str, Any]]" = []
        for conf in file_conf:
            KubernetesLocalGraph._extract_nested_resources_recursive(conf, all_resources)
        return all_resources
            
    @staticmethod
    def _extract_nested_resources_recursive(conf: Dict[str, Any], all_resources: List[Dict[str, Any]]) -> None:
        spec = conf.get('spec')
        if not spec or not isinstance(spec, dict):
            all_resources.append(conf)
            return
        template = spec.get('template', None)
        if not template or not isinstance(template, dict):
            all_resources.append(conf)
            return
        if conf.get('kind') in SUPPORTED_POD_CONTAINERS_TYPES:
            # means this is a Pod resource nested in a supported template container resource
            template[PARENT_RESOURCE_ID_KEY_NAME] = get_resource_id(conf)
            template[PARENT_RESOURCE_KEY_NAME] = conf.get('metadata', {}).get('name', "")
        spec.pop('template', None)
        all_resources.append(conf)
        KubernetesLocalGraph._extract_nested_resources_recursive(template, all_resources)

    def update_vertices_configs(self) -> None:
        # not used
        return None

    @staticmethod
    def update_vertex_config(vertex: KubernetesBlock, changed_attributes: list[str] | dict[str, Any], dynamic_blocks: bool = False) -> None:
        # not used
        return None

    def get_resources_types_in_graph(self) -> list[str]:
        # not used
        return []



import datetime
import json
from typing import Any

from lark import Tree
from bc_jsonpath_ng import parse, JSONPath

from checkov.common.bridgecrew.severities import Severity
from checkov.common.output.common import ImageDetails
from checkov.common.packaging.version import LegacyVersion, Version


class CustomJSONEncoder(json.JSONEncoder):
    def default(self, o: Any) -> Any:
        if isinstance(o, set):
            return list(o)
        elif isinstance(o, Tree):
            return str(o)
        elif isinstance(o, datetime.date):
            return str(o)
        elif isinstance(o, (Version, LegacyVersion)):
            return str(o)
        elif isinstance(o, Severity):
            return o.name
        elif isinstance(o, complex):
            return str(o)
        elif isinstance(o, ImageDetails):
            return o.__dict__
        else:
            return json.JSONEncoder.default(self, o)


def get_jsonpath_from_evaluated_key(evaluated_key: str) -> JSONPath:
    evaluated_key = evaluated_key.replace("/", ".")
    return parse(f"$..{evaluated_key}")  # type:ignore[no-any-return]



from __future__ import annotations

from typing import TYPE_CHECKING

from checkov.common.models.enums import CheckCategories, CheckResult
from checkov.dockerfile.base_dockerfile_check import BaseDockerfileCheck

if TYPE_CHECKING:
    from dockerfile_parse.parser import _Instruction


class RunUsingAPT(BaseDockerfileCheck):
    def __init__(self) -> None:
        """
        Apt interface is less stable than apt-get and so this preferred
        """
        name = "Ensure that APT isn't used"
        id = "CKV_DOCKER_9"
        supported_instructions = ("RUN",)
        categories = (CheckCategories.NETWORKING,)
        super().__init__(name=name, id=id, categories=categories, supported_instructions=supported_instructions)

    def scan_resource_conf(self, conf: list[_Instruction]) -> tuple[CheckResult, list[_Instruction] | None]:
        for run in conf:
            content = run["content"]
            if " apt " in content:
                return CheckResult.FAILED, [run]
        return CheckResult.PASSED, None


check = RunUsingAPT()



from enum import Enum


class ValidationStatus(Enum):
    PRIVILEGED = 'Privileged'
    VALID = 'Valid'
    INVALID = 'Invalid'
    UNKNOWN = 'Unknown'

    def __str__(self) -> str:
        # needed, because of a Python 3.11 change
        return self.value


class VerifySecretsResult(Enum):
    INSUFFICIENT_PARAMS = 'INSUFFICIENT_PARAMS'
    FAILURE = 'FAILURE'
    SUCCESS = 'SUCESS'

    def __str__(self) -> str:
        # needed, because of a Python 3.11 change
        return self.value



from __future__ import annotations

from typing import Any, TYPE_CHECKING

from checkov.common.graph.checks_infra.enums import Operators
from checkov.common.checks_infra.solvers.attribute_solvers.base_attribute_solver import BaseAttributeSolver
from checkov.common.util.type_forcers import force_int

if TYPE_CHECKING:
    from typing_extensions import TypeGuard


class BaseNumberOfWordsAttributeSolver(BaseAttributeSolver):
    operator = Operators.NUMBER_OF_WORDS_GREATER_THAN  # noqa: CCE003  # a static attribute

    def _validate_vertex_value(self, attr: Any) -> TypeGuard[str]:
        return isinstance(attr, str)

    def _get_number_of_words(self, attr: str) -> int:
        return len(attr.split())

    def _numerize_value(self) -> int | None:
        return force_int(self.value)

    def _get_operation(self, vertex: dict[str, Any], attribute: str | None) -> bool:
        attr = vertex.get(attribute)  # type:ignore[arg-type]  # due to attribute can be None
        return self._validate_vertex_value(attr)



from __future__ import annotations

from typing import Any

from checkov.common.models.enums import CheckResult, CheckCategories
from checkov.common.util.type_forcers import force_list, extract_policy_dict
from checkov.terraform.checks.resource.base_resource_check import BaseResourceCheck


class IAMAdminPolicyDocument(BaseResourceCheck):
    def __init__(self) -> None:
        name = 'Ensure IAM policies that allow full "*-*" administrative privileges are not created'
        id = "CKV_AWS_62"
        supported_resources = (
            "aws_iam_role_policy",
            "aws_iam_user_policy",
            "aws_iam_group_policy",
            "aws_iam_policy",
            "aws_ssoadmin_permission_set_inline_policy",
        )
        categories = (CheckCategories.IAM,)
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def scan_resource_conf(self, conf: dict[str, list[Any]]) -> CheckResult:
        if "policy" in conf.keys():
            policy_str = "policy"
        elif "inline_policy" in conf.keys():
            policy_str = "inline_policy"
        else:
            return CheckResult.PASSED

        try:
            policy_block = extract_policy_dict(conf[policy_str][0])
            if policy_block and "Statement" in policy_block.keys():
                for statement in force_list(policy_block["Statement"]):
                    if "Action" in statement:
                        effect = statement.get("Effect", "Allow")
                        action = force_list(statement.get("Action", [""]))
                        resource = force_list(statement.get("Resource", [""]))
                        if effect == "Allow" and "*" in action and "*" in resource:
                            return CheckResult.FAILED
        except Exception:  # nosec
            pass
        return CheckResult.PASSED

    def get_evaluated_keys(self) -> list[str]:
        return ["policy", "inline_policy"]


check = IAMAdminPolicyDocument()



from typing import Optional, Any, Dict
from collections.abc import Sized
from checkov.common.checks_infra.solvers.attribute_solvers.base_attribute_solver import BaseAttributeSolver
from checkov.common.graph.checks_infra.enums import Operators
from checkov.common.util.type_forcers import force_int


class LengthEqualsAttributeSolver(BaseAttributeSolver):
    operator = Operators.LENGTH_EQUALS  # noqa: CCE003  # a static attribute

    def _get_operation(self, vertex: Dict[str, Any], attribute: Optional[str]) -> bool:
        if vertex.get(attribute) is None:  # type:ignore[arg-type]  # due to attribute can be None
            return False

        attr = vertex.get(attribute)  # type:ignore[arg-type]  # due to attribute can be None
        if isinstance(attr, Sized):
            return len(attr) == force_int(self.value)

        return False



from checkov.common.models.enums import CheckCategories, CheckResult
from checkov.terraform.checks.resource.base_resource_check import BaseResourceCheck


class DiskIsEncrypted(BaseResourceCheck):
    def __init__(self):
        name = "Ensure disk is encrypted"
        id = "CKV_ALI_7"
        supported_resources = ['alicloud_disk']
        categories = [CheckCategories.ENCRYPTION]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def scan_resource_conf(self, conf):
        if conf.get("snapshot_id"):
            return CheckResult.UNKNOWN
        if conf.get("encrypted") and conf.get("encrypted") == [True]:
            return CheckResult.PASSED
        return CheckResult.FAILED


check = DiskIsEncrypted()



# NOTE: Leaving this around for backwards compatibility. Usage of the checks/function/base_function_check.py
#       version is preferred
# noinspection PyUnresolvedReferences
from checkov.serverless.checks.function.base_function_check import BaseFunctionCheck  # noqa



from __future__ import annotations
from typing import Any

from checkov.common.images.image_referencer import Image
from checkov.circleci_pipelines.image_referencer.provider import CircleCIProvider


class CircleCIImageReferencerManager:
    __slots__ = ("workflow_config", "file_path")

    def __init__(self, workflow_config: dict[str, Any], file_path: str) -> None:
        self.workflow_config = workflow_config
        self.file_path = file_path

    def extract_images_from_workflow(self) -> list[Image]:
        provider = CircleCIProvider(workflow_config=self.workflow_config, file_path=self.file_path)
        images: list[Image] = provider.extract_images_from_workflow()

        return images



from __future__ import annotations

import json
import re
from typing import Generator
from typing import Any
from typing import TYPE_CHECKING
from typing import Pattern

from detect_secrets.plugins.high_entropy_strings import Base64HighEntropyString
from detect_secrets.plugins.high_entropy_strings import HexHighEntropyString
from detect_secrets.plugins.keyword import KeywordDetector
from detect_secrets.plugins.keyword import DENYLIST
from detect_secrets.plugins.keyword import AFFIX_REGEX
from detect_secrets.plugins.keyword import CLOSING
from detect_secrets.plugins.keyword import OPTIONAL_WHITESPACE
from detect_secrets.plugins.keyword import QUOTE
from detect_secrets.plugins.keyword import SECRET
from detect_secrets.plugins.base import BasePlugin
from detect_secrets.util.filetype import FileType
from detect_secrets.util.filetype import determine_file_type

from checkov.secrets.parsers.terraform.multiline_parser import terraform_multiline_parser
from checkov.secrets.parsers.terraform.single_line_parser import terraform_single_line_parser
from checkov.secrets.runner import SOURCE_CODE_EXTENSION
from checkov.secrets.parsers.yaml.multiline_parser import yml_multiline_parser
from checkov.secrets.parsers.json.multiline_parser import json_multiline_parser

if TYPE_CHECKING:
    from checkov.secrets.parsers.multiline_parser import BaseMultiLineParser
    from detect_secrets.core.potential_secret import PotentialSecret
    from detect_secrets.util.code_snippet import CodeSnippet

MAX_LINE_LENGTH = 10000
MAX_KEYWORD_LIMIT = 500
ENTROPY_KEYWORD_COMBINATOR_LIMIT = 3
ENTROPY_KEYWORD_LIMIT = 4.5


DENY_LIST_REGEX = r'|'.join(DENYLIST)
# Support for suffix after keyword i.e. password_secure = "value"
DENY_LIST_REGEX2 = r'({denylist}){suffix}'.format(
    denylist=DENY_LIST_REGEX,
    suffix=AFFIX_REGEX,
)

KEY = r'{words}({closing})?'.format(
    words=AFFIX_REGEX,
    closing=CLOSING,
)

FOLLOWED_BY_COLON_VALUE_KEYWORD_REGEX = re.compile(
    # e.g. var: MY_PASSWORD_123
    r'{whitespace}({key})?:{whitespace}({quote}?){words}{denylist}({closing})?(\3)'.format(
        key=KEY,
        whitespace=OPTIONAL_WHITESPACE,
        quote=QUOTE,
        words=AFFIX_REGEX,
        denylist=DENY_LIST_REGEX2,
        closing=CLOSING,
    ),
    flags=re.IGNORECASE,
)

QUOTES_REQUIRED_FOLLOWED_BY_COLON_VALUE_KEYWORD_REGEX = re.compile(
    # e.g. var: MY_PASSWORD_123
    r'{whitespace}"({key})?":{whitespace}("?){words}{denylist}({closing})?(\3)'.format(
        key=KEY,
        whitespace=OPTIONAL_WHITESPACE,
        words=AFFIX_REGEX,
        denylist=DENY_LIST_REGEX2,
        closing=CLOSING,
    ),
    flags=re.IGNORECASE,
)

FOLLOWED_BY_COLON_VALUE_SECRET_REGEX = re.compile(
    # e.g. var: Zmlyc3Rfc2VjcmV0X2hlcmVfd2hvYV9tdWx0aWxsaW5lX3Nob3VsZF93b3JrXzE==
    r'{whitespace}({key})?:{whitespace}({quote}?)({secret})(\3)'.format(
        key=KEY,
        whitespace=OPTIONAL_WHITESPACE,
        quote=QUOTE,
        secret=SECRET,
    ),
    flags=re.IGNORECASE,
)

QUOTES_REQUIRED_FOLLOWED_BY_COLON_VALUE_SECRET_REGEX = re.compile(
    # e.g. var: Zmlyc3Rfc2VjcmV0X2hlcmVfd2hvYV9tdWx0aWxsaW5lX3Nob3VsZF93b3JrXzE==
    r'{whitespace}"({key})?":{whitespace}("?)({secret})(\3)'.format(
        key=KEY,
        whitespace=OPTIONAL_WHITESPACE,
        secret=SECRET,
    ),
    flags=re.IGNORECASE,
)

FOLLOWED_BY_EQUAL_VALUE_KEYWORD_REGEX = re.compile(
    # e.g. var = MY_PASSWORD_123
    r'{whitespace}({key})?={whitespace}({quote}?){words}{denylist}({closing})?(\3)'.format(
        key=KEY,
        whitespace=OPTIONAL_WHITESPACE,
        quote=QUOTE,
        words=AFFIX_REGEX,
        denylist=DENY_LIST_REGEX2,
        closing=CLOSING,
    ),
    flags=re.IGNORECASE,
)

FOLLOWED_BY_EQUAL_VALUE_SECRET_REGEX = re.compile(
    # e.g. var = Zmlyc3Rfc2VjcmV0X2hlcmVfd2hvYV9tdWx0aWxsaW5lX3Nob3VsZF93b3JrXzE==
    r'{whitespace}({key})?={whitespace}({quote}?)({secret})(\3)'.format(
        key=KEY,
        whitespace=OPTIONAL_WHITESPACE,
        quote=QUOTE,
        secret=SECRET,
    ),
    flags=re.IGNORECASE,
)

#  if the current regex is not enough, can add more regexes to check

YML_PAIR_VALUE_KEYWORD_REGEX_TO_GROUP = {
    FOLLOWED_BY_COLON_VALUE_KEYWORD_REGEX: 4,
}

YML_PAIR_VALUE_SECRET_REGEX_TO_GROUP = {
    FOLLOWED_BY_COLON_VALUE_SECRET_REGEX: 4,
}

JSON_PAIR_VALUE_KEYWORD_REGEX_TO_GROUP = {
    QUOTES_REQUIRED_FOLLOWED_BY_COLON_VALUE_KEYWORD_REGEX: 4,
}

JSON_PAIR_VALUE_SECRET_REGEX_TO_GROUP = {
    QUOTES_REQUIRED_FOLLOWED_BY_COLON_VALUE_SECRET_REGEX: 4,
}

TERRAFORM_PAIR_VALUE_KEYWORD_REGEX_TO_GROUP = {
    FOLLOWED_BY_EQUAL_VALUE_KEYWORD_REGEX: 4,
}

TERRAFORM_PAIR_VALUE_SECRET_REGEX_TO_GROUP = {
    FOLLOWED_BY_EQUAL_VALUE_SECRET_REGEX: 4,
}

REGEX_VALUE_KEYWORD_BY_FILETYPE = {
    FileType.YAML: YML_PAIR_VALUE_KEYWORD_REGEX_TO_GROUP,
    FileType.JSON: JSON_PAIR_VALUE_KEYWORD_REGEX_TO_GROUP,
    FileType.TERRAFORM: TERRAFORM_PAIR_VALUE_KEYWORD_REGEX_TO_GROUP,
}

REGEX_VALUE_SECRET_BY_FILETYPE = {
    FileType.YAML: YML_PAIR_VALUE_SECRET_REGEX_TO_GROUP,
    FileType.JSON: JSON_PAIR_VALUE_SECRET_REGEX_TO_GROUP,
    FileType.TERRAFORM: TERRAFORM_PAIR_VALUE_SECRET_REGEX_TO_GROUP,
}

SINGLE_LINE_PARSER = {
    FileType.TERRAFORM: terraform_single_line_parser,
}

MULTILINE_PARSERS = {
    FileType.YAML: (
        (FileType.YAML, yml_multiline_parser),
    ),
    FileType.JSON: (
        (FileType.JSON, json_multiline_parser),
    ),
    FileType.TERRAFORM: (
        (FileType.TERRAFORM, terraform_multiline_parser),
        (FileType.JSON, json_multiline_parser),
        (FileType.YAML, yml_multiline_parser),
    ),
}


class EntropyKeywordCombinator(BasePlugin):
    secret_type = ""  # nosec  # noqa: CCE003  # a static attribute

    def __init__(self, limit: float = ENTROPY_KEYWORD_LIMIT) -> None:
        iac_limit = ENTROPY_KEYWORD_COMBINATOR_LIMIT
        self.high_entropy_scanners_iac = (Base64HighEntropyString(limit=iac_limit), HexHighEntropyString(limit=iac_limit))
        self.high_entropy_scanners = (Base64HighEntropyString(limit=limit), HexHighEntropyString(limit=limit))
        self.keyword_scanner = KeywordDetector()

    def analyze_string(self, string: str) -> Generator[str, None, None]:
        yield ""

    def analyze_line(
            self,
            filename: str,
            line: str,
            line_number: int = 0,
            context: CodeSnippet | None = None,
            raw_context: CodeSnippet | None = None,
            **kwargs: Any,
    ) -> set[PotentialSecret]:
        if len(line) > MAX_LINE_LENGTH:
            # to keep good performance we skip long lines
            return set()

        is_iac = f".{filename.split('.')[-1]}" not in SOURCE_CODE_EXTENSION
        keyword_on_key = self.keyword_scanner.analyze_line(filename, line, line_number, **kwargs)
        if is_iac:
            filetype = determine_file_type(filename)
            single_line_parser = SINGLE_LINE_PARSER.get(filetype)
            multiline_parsers = MULTILINE_PARSERS.get(filetype)

            # classic key-value pair
            if keyword_on_key:
                if single_line_parser:
                    return single_line_parser.detect_secret(
                        scanners=self.high_entropy_scanners_iac,
                        filename=filename,
                        raw_context=raw_context,
                        line=line,
                        line_number=line_number,
                        kwargs=kwargs
                    )
                else:
                    for pt in keyword_on_key:
                        if pt.secret_value:
                            quoted_secret = f"\"{pt.secret_value}\""
                            if line.find(quoted_secret) < 0:    # replace potential secret with quoted version
                                line = line.replace(pt.secret_value, f"\"{pt.secret_value}\"", 1)
                    return self.detect_secret(
                        scanners=self.high_entropy_scanners_iac,
                        filename=filename,
                        line=line,
                        line_number=line_number,
                        kwargs=kwargs
                    )

            # not so classic key-value pair, from multiline, that is only in an array format.
            # The scan searches forwards and backwards for a potential secret pair, so no duplicates expected.
            elif multiline_parsers:
                # iterate over multiple parser and their related file type.
                # this is needed for file types, which embed other file type parser, ex Terraform with heredoc
                for parser_file_type, multiline_parser in multiline_parsers:
                    value_keyword_regex_to_group = REGEX_VALUE_KEYWORD_BY_FILETYPE.get(parser_file_type)
                    secret_keyword_regex_to_group = REGEX_VALUE_SECRET_BY_FILETYPE.get(parser_file_type)

                    potential_secrets = self.analyze_multiline(
                        filename=filename,
                        line=line,
                        multiline_parser=multiline_parser,
                        line_number=line_number,
                        context=context,
                        raw_context=raw_context,
                        value_pattern=value_keyword_regex_to_group,
                        secret_pattern=secret_keyword_regex_to_group,
                        kwargs=kwargs
                    )

                    if potential_secrets:
                        # return a possible secret, otherwise check with next parser
                        return potential_secrets
        else:
            return self.detect_secret(
                # If we found a keyword (i.e. db_pass = ), lower the threshold to the iac threshold
                scanners=self.high_entropy_scanners if not keyword_on_key else self.high_entropy_scanners_iac,
                filename=filename,
                line=line,
                line_number=line_number,
                kwargs=kwargs
            )

        return set()

    def analyze_multiline(
            self,
            filename: str,
            line: str,
            multiline_parser: BaseMultiLineParser,
            line_number: int = 0,
            context: CodeSnippet | None = None,
            raw_context: CodeSnippet | None = None,
            value_pattern: dict[Pattern[str], int] | None = None,
            secret_pattern: dict[Pattern[str], int] | None = None,
            **kwargs: Any,
    ) -> set[PotentialSecret]:
        secrets: set[PotentialSecret] = set()
        if context is None or raw_context is None:
            return secrets
        value_secrets = self.extract_from_string(pattern=secret_pattern, string=context.target_line)
        for possible_secret in value_secrets:
            secret_adjust = self.format_reducing_noise_secret(possible_secret)

            entropy_on_value = self.detect_secret(
                scanners=self.high_entropy_scanners,
                filename=filename,
                line=secret_adjust,
                line_number=line_number,
                kwargs=kwargs
            )

            if entropy_on_value:
                possible_keywords: set[str] = set()
                backwards_range = range(context.target_index - 1, -1, -1)
                forward_range = range(context.target_index + 1, len(context.lines))

                possible_keywords |= multiline_parser.get_lines_from_same_object(
                    search_range=forward_range,
                    context=context,
                    raw_context=raw_context,
                    line_length_limit=MAX_KEYWORD_LIMIT)
                possible_keywords |= multiline_parser.get_lines_from_same_object(
                    search_range=backwards_range,
                    context=context,
                    raw_context=raw_context,
                    line_length_limit=MAX_KEYWORD_LIMIT)

                for other_value in possible_keywords:
                    if self.extract_from_string(pattern=value_pattern, string=other_value):
                        secrets |= entropy_on_value
                        break
        return secrets

    @staticmethod
    def format_reducing_noise_secret(string: str) -> str:
        return json.dumps(string)

    @staticmethod
    def extract_from_string(pattern: dict[Pattern[str], int] | None, string: str) -> set[str]:
        matches: set[str] = set()
        if not pattern:
            return matches
        for value_regex, group_number in pattern.items():
            match = value_regex.search(string)
            if match:
                matches |= {match.group(group_number)}
        return matches

    @staticmethod
    def detect_secret(
            scanners: tuple[Base64HighEntropyString, HexHighEntropyString],
            filename: str,
            line: str,
            line_number: int = 0,
            **kwargs: Any,
    ) -> set[PotentialSecret]:
        for entropy_scanner in scanners:
            matches = entropy_scanner.analyze_line(filename, line, line_number, **kwargs)
            if matches:
                return matches
        return set()



from __future__ import annotations

import logging
import os
import platform

from typing import Type, Optional

from checkov.common.graph.checks_infra.registry import BaseRegistry
from checkov.common.graph.db_connectors.networkx.networkx_db_connector import NetworkxConnector
from checkov.terraform.graph_builder.graph_components.block_types import BlockType
from checkov.terraform.graph_manager import TerraformGraphManager
from checkov.terraform.graph_builder.local_graph import TerraformLocalGraph
from checkov.common.checks_infra.registry import get_graph_checks_registry
from checkov.common.graph.graph_builder.graph_components.attribute_names import CustomAttributes
from checkov.common.output.record import Record
from checkov.common.util.secrets import omit_secret_value_from_checks

from checkov.common.bridgecrew.check_type import CheckType
from checkov.common.output.report import Report
from checkov.common.runners.base_runner import CHECKOV_CREATE_GRAPH
from checkov.runner_filter import RunnerFilter
from checkov.terraform.checks.resource.registry import resource_registry
from checkov.terraform.context_parsers.registry import parser_registry
from checkov.terraform.plan_utils import create_definitions, build_definitions_context, \
    get_resource_id_without_nested_modules
from checkov.terraform.runner import Runner as TerraformRunner, merge_reports
from checkov.terraform.deep_analysis_plan_graph_manager import DeepAnalysisGraphManager


# set of check IDs with lifecycle condition
TF_LIFECYCLE_CHECK_IDS = {
    "CKV_AWS_217",
    "CKV_AWS_233",
    "CKV_AWS_237",
    "CKV_GCP_82",
}

RESOURCE_ATTRIBUTES_TO_OMIT = {
    'aws_db_instance': ['password'],
    'aws_secretsmanager_secret_version': ['secret_string'],
    'aws_ssm_parameter': ['value'],
    'azurerm_container_registry': ['admin_password'],
    'azurerm_key_vault_secret': ['value'],
    'azurerm_linux_virtual_machine': ['admin_password'],
    'azurerm_mssql_managed_instance_vulnerability_assessment': ['storage_container_path'],
    'azurerm_mssql_server': ['administrator_login_password'],
    'azurerm_mssql_server_vulnerability_assessment': ['storage_container_path'],
    'azurerm_redis_cache': ['primary_access_key', 'secondary_access_key', 'primary_connection_string',
                            'secondary_connection_string'],
    'azurerm_sql_server': ['administrator_login_password'],
    'azurerm_sql_managed_instance': ['administrator_login_password'],
    'azurerm_storage_account': ['primary_access_key', 'secondary_access_key', 'primary_blob_connection_string',
                                'secondary_blob_connection_string', 'primary_blob_endpoint', 'primary_blob_host',
                                'secondary_blob_endpoint', 'secondary_blob_host', 'primary_connection_string',
                                'secondary_connection_string'],
    'azurerm_synapse_workspace_vulnerability_assessment': ['storage_container_path'],
    'azurerm_synapse_sql_pool_vulnerability_assessment': ['storage_container_path'],
    'azurerm_virtual_machine': ['admin_password'],
    'azurerm_windows_virtual_machine': ['admin_password'],
    'google_kms_secret_ciphertext': ['plaintext']
}


class Runner(TerraformRunner):
    check_type = CheckType.TERRAFORM_PLAN  # noqa: CCE003  # a static attribute

    def __init__(self, graph_class: Type[TerraformLocalGraph] = TerraformLocalGraph,
                 graph_manager: TerraformGraphManager | None = None,
                 db_connector: NetworkxConnector | None = None,
                 external_registries: list[BaseRegistry] | None = None,
                 source: str = "Terraform") -> None:
        super().__init__(
            graph_class=graph_class,
            graph_manager=graph_manager,
            db_connector=db_connector or NetworkxConnector(),
            external_registries=external_registries,
            source=source,
        )
        self.file_extensions = ['.json']  # override what gets set from the TF runner
        self.definitions = None
        self.context = None
        self.graph_registry = get_graph_checks_registry(super().check_type)
        self.deep_analysis = False
        self.repo_root_for_plan_enrichment = []
        self.tf_plan_local_graph = None

    block_type_registries = {  # noqa: CCE003  # a static attribute
        'resource': resource_registry,
    }

    def run(
            self,
            root_folder: str | None = None,
            external_checks_dir: list[str] | None = None,
            files: list[str] | None = None,
            runner_filter: RunnerFilter | None = None,
            collect_skip_comments: bool = True
    ) -> Report | list[Report]:
        runner_filter = runner_filter or RunnerFilter()
        # Update resource_attr_to_omit according to plan runner hardcoded RESOURCE_ATTRIBUTES_TO_OMIT
        self._extend_resource_attributes_to_omit(runner_filter)
        self.deep_analysis = runner_filter.deep_analysis
        if runner_filter.repo_root_for_plan_enrichment:
            self.repo_root_for_plan_enrichment = os.path.abspath(runner_filter.repo_root_for_plan_enrichment[0])
        report = Report(self.check_type)
        parsing_errors: dict[str, str] = {}
        tf_local_graph: Optional[TerraformLocalGraph] = None
        if self.definitions is None or self.context is None:
            self.definitions, definitions_raw = create_definitions(root_folder, files, runner_filter, parsing_errors)
            self.context = build_definitions_context(self.definitions, definitions_raw)
            if CHECKOV_CREATE_GRAPH:
                self.tf_plan_local_graph = self.graph_manager.build_graph_from_definitions(self.definitions, render_variables=False)
                for vertex in self.tf_plan_local_graph.vertices:
                    if vertex.block_type == BlockType.RESOURCE:
                        address = vertex.attributes.get(CustomAttributes.TF_RESOURCE_ADDRESS)
                        if self.enable_nested_modules:
                            report.add_resource(f'{vertex.path}:{address}')
                        else:
                            resource_id = get_resource_id_without_nested_modules(address)
                            report.add_resource(f'{vertex.path}:{resource_id}')
                self.graph_manager.save_graph(self.tf_plan_local_graph)
                if self._should_run_deep_analysis:
                    tf_local_graph = self._create_terraform_graph()

        if external_checks_dir:
            for directory in external_checks_dir:
                resource_registry.load_external_checks(directory)
                self.graph_registry.load_external_checks(directory)
        if not root_folder:
            root_folder = os.path.split(os.path.commonprefix(files))[0]
        self.check_tf_definition(report, root_folder, runner_filter)
        report.add_parsing_errors(parsing_errors.keys())

        if self.definitions:
            graph_report = self._get_graph_report(
                root_folder=root_folder,
                runner_filter=runner_filter,
                tf_local_graph=tf_local_graph
            )
            merge_reports(report, graph_report)

        if runner_filter.run_image_referencer:
            image_report = self.check_container_image_references(
                graph_connector=self.graph_manager.get_reader_endpoint(),
                root_path=root_folder,
                runner_filter=runner_filter,
            )

            if image_report:
                # due too many tests failing only return a list, if there is an image report
                return [report, image_report]

        return report

    @staticmethod
    def _extend_resource_attributes_to_omit(runner_filter: RunnerFilter):
        for k, v in RESOURCE_ATTRIBUTES_TO_OMIT.items():
            # It's ok as runner_filter is ALWAYS default dict with set() as value
            runner_filter.resource_attr_to_omit[k].update(v)

    def _get_graph_report(
            self,
            root_folder: str,
            runner_filter: RunnerFilter,
            tf_local_graph: Optional[TerraformLocalGraph]
    ) -> Report:
        if self._should_run_deep_analysis and tf_local_graph:
            deep_analysis_graph_manager = DeepAnalysisGraphManager(tf_local_graph, self.tf_plan_local_graph)
            deep_analysis_graph_manager.enrich_tf_graph_attributes()
            self.graph_manager.save_graph(tf_local_graph)
            graph_report = self.get_graph_checks_report(root_folder, runner_filter)
            deep_analysis_graph_manager.filter_report(graph_report)
            return graph_report
        return self.get_graph_checks_report(root_folder, runner_filter)

    def _create_terraform_graph(self) -> TerraformLocalGraph:
        graph_manager = TerraformGraphManager(db_connector=NetworkxConnector())
        tf_local_graph, _ = graph_manager.build_graph_from_source_directory(
            self.repo_root_for_plan_enrichment,
            render_variables=True
        )
        self.graph_manager = graph_manager
        return tf_local_graph

    def check_tf_definition(self, report, root_folder, runner_filter, collect_skip_comments=True):
        for full_file_path, definition in self.definitions.items():
            if platform.system() == "Windows":
                temp = os.path.split(full_file_path)[0]
                scanned_file = f"/{os.path.relpath(full_file_path,temp)}"
            else:
                scanned_file = f"/{os.path.relpath(full_file_path, root_folder)}"
            logging.debug(f"Scanning file: {scanned_file}")
            for block_type in definition.keys():
                if block_type in self.block_type_registries.keys():
                    self.run_block(definition[block_type], None, full_file_path, root_folder, report, scanned_file,
                                   block_type, runner_filter)

    def run_block(self, entities,
                  definition_context,
                  full_file_path, root_folder, report, scanned_file,
                  block_type, runner_filter=None, entity_context_path_header=None,
                  module_referrer: str | None = None):
        registry = self.block_type_registries[block_type]
        if registry:
            for entity in entities:
                context_parser = parser_registry.context_parsers[block_type]
                definition_path = context_parser.get_entity_context_path(entity)
                # Entity can exist only once per dir, for file as well
                entity_context = self.get_entity_context(definition_path, full_file_path)
                entity_lines_range = [entity_context.get('start_line'), entity_context.get('end_line')]
                entity_code_lines = entity_context.get('code_lines')
                entity_address = entity_context.get('address')
                entity_id = entity_address if self.enable_nested_modules else get_resource_id_without_nested_modules(entity_address)
                _, _, entity_config = registry.extract_entity_details(entity)

                results = registry.scan(scanned_file, entity, [], runner_filter, report_type=CheckType.TERRAFORM_PLAN)
                for check, check_result in results.items():
                    if check.id in TF_LIFECYCLE_CHECK_IDS:
                        # can't be evaluated in TF plan
                        continue
                    censored_code_lines = omit_secret_value_from_checks(
                        check=check,
                        check_result=check_result,
                        entity_code_lines=entity_code_lines,
                        entity_config=entity_config,
                        resource_attributes_to_omit=runner_filter.resource_attr_to_omit
                    )
                    record = Record(
                        check_id=check.id,
                        bc_check_id=check.bc_id,
                        check_name=check.name,
                        check_result=check_result,
                        code_block=censored_code_lines,
                        file_path=scanned_file,
                        file_line_range=entity_lines_range,
                        resource=entity_id,
                        resource_address=entity_address,
                        evaluations=None,
                        check_class=check.__class__.__module__,
                        file_abs_path=full_file_path,
                        severity=check.severity,
                        details=check.details,
                    )
                    record.set_guideline(check.guideline)
                    report.add_record(record=record)

    def get_entity_context_and_evaluations(self, entity):
        raw_context = self.get_entity_context(entity[CustomAttributes.BLOCK_NAME].split("."),
                                              entity[CustomAttributes.FILE_PATH])
        raw_context['definition_path'] = entity[CustomAttributes.BLOCK_NAME].split('.')
        return raw_context, None

    def get_entity_context(self, definition_path, full_file_path):
        entity_id = ".".join(definition_path)
        return self.context.get(full_file_path, {}).get(entity_id)

    @property
    def _should_run_deep_analysis(self) -> bool:
        return self.deep_analysis and self.repo_root_for_plan_enrichment and self.tf_plan_local_graph



from checkov.terraform.checks.resource.base_resource_value_check import BaseResourceValueCheck
from checkov.common.models.enums import CheckCategories


class ECRImageScanning(BaseResourceValueCheck):
    def __init__(self):
        name = "Ensure ECR image scanning on push is enabled"
        id = "CKV_AWS_163"
        supported_resources = ['aws_ecr_repository']
        categories = [CheckCategories.GENERAL_SECURITY]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def get_inspected_key(self):
        return "image_scanning_configuration/[0]/scan_on_push"


check = ECRImageScanning()



from __future__ import annotations

import itertools
from typing import Optional, List, Tuple, Dict, Any, TYPE_CHECKING

from checkov.common.checks_infra.solvers.connections_solvers.base_connection_solver import BaseConnectionSolver
from checkov.common.graph.checks_infra.enums import Operators
from checkov.common.graph.checks_infra.solvers.base_solver import BaseSolver
from checkov.common.checks_infra.solvers.connections_solvers.complex_connection_solver import ComplexConnectionSolver

if TYPE_CHECKING:
    from networkx import DiGraph


class OrConnectionSolver(ComplexConnectionSolver):
    operator = Operators.OR  # noqa: CCE003  # a static attribute

    def __init__(self, solvers: Optional[List[BaseSolver]], operator: str) -> None:
        super().__init__(solvers, operator)

    def get_operation(self, graph_connector: DiGraph) -> \
            Tuple[List[Dict[str, Any]], List[Dict[str, Any]], List[Dict[str, Any]]]:
        passed, failed, unknown = self.run_attribute_solvers(graph_connector)
        connection_solvers = [sub_solver for sub_solver in self.solvers if isinstance(sub_solver, BaseConnectionSolver)]
        for connection_solver in connection_solvers:
            connection_solver.set_vertices(graph_connector, [], [])
            passed_solver, failed_solver, unknown_solver = connection_solver.get_operation(graph_connector)
            passed.extend(passed_solver)
            failed.extend(failed_solver)
            unknown.extend(unknown_solver)

        passed_path_and_ids = [(ComplexConnectionSolver.get_check_identifier(p)) for p in passed]
        unknown_path_and_ids = [(ComplexConnectionSolver.get_check_identifier(u)) for u in unknown]
        unknown = [u for u in unknown if (ComplexConnectionSolver.get_check_identifier(u)) not in passed_path_and_ids]
        failed = [f for f in failed if (ComplexConnectionSolver.get_check_identifier(f)) not in itertools.chain(passed_path_and_ids, unknown_path_and_ids)]
        return self.filter_results(passed, failed, unknown)
